#include "Timers.as"

const int BITE_TIME = 10;
const float BITE_DISTANCE = 16.0f;

void onInit( CBlob@ this )
{
	this.getShape().SetRotationsAllowed(false);
	this.getShape().getConsts().transports = true;
	this.SetMapEdgeFlags(u8(CBlob::map_collide_none));
}

void onTick( CBlob@ this )
{
	if (!isBiting(this))
	{
		// swim

		Vec2f vel = this.getVelocity();
		if (this.isFacingLeft()){
			if (vel.x > -1.0f){
				vel.x -= 0.05f;
			}
		}
		else {
			if (vel.x < 1.0f){
				vel.x += 0.05f;
			}
		}
		this.setVelocity(vel);

		CSprite@ sprite = this.getSprite();
		// bite
		if (sprite.isAnimation("bite")){
			sprite.SetAnimation("default");
			sprite.PlaySound("Chomp");

			AttachmentPoint @ap = this.getAttachments().getAttachmentPointByName("MOUTH");
			if (ap !is null)
			{
				CBlob@[] blobsInRadius;
				if (this.getMap().getBlobsInRadius(ap.getPosition(), BITE_DISTANCE, @blobsInRadius))
				{
					for (uint i = 0; i < blobsInRadius.length; i++)
					{
						CBlob @b = blobsInRadius[i];
						if (b.hasTag("player"))
						{
							sprite.PlaySound("FleshEat");
							this.server_Hit(b, b.getPosition(),
			               	 b.getPosition() - this.getPosition(), 3.0f,
			               	 0, true);
						}
					}
				}
			}
		}
	}
}

void onCollision(CBlob@ this, CBlob@ blob, bool solid, Vec2f normal, Vec2f point1)
{
	Vec2f pos = this.getPosition();
	if (blob is null) 
	{
		if (this.isFacingLeft() && point1.x < pos.x){
			this.SetFacingLeft(false);
		}
		else if (point1.x > pos.x) {
			this.SetFacingLeft(true);
		}
		return;
	}

	if (blob.hasTag("player"))
	{
		if (!isBiting(this))
		{
			// turn around
			if ((!this.isFacingLeft() && point1.x > pos.x) 
				|| (this.isFacingLeft() && point1.x < pos.x)) {
				this.set_u32("croc bite time", getGameTime());
				CSprite@ sprite = this.getSprite();
				sprite.SetAnimation("bite");
				sprite.PlaySound("StartChomp");
			}
		}
	}
}

bool isBiting( CBlob@ this )
{
	return getGameTime() - this.get_u32("croc bite time") < BITE_TIME;
}