#include "PokerCommon.as"
#include "UI.as"
#include "DebugButton.as"
#include "RulesCommon.as"
#include "SoldierCommon.as"
#include "GameColours.as"

const string BAR_MENU = "barmenu";
int _leaderboardShowtime = 0;
const int LEADERBOARD_TIME = 90;
string _score_spritesheet = "Sprites/UI/hud_scores.png";

void onInit(CBlob@ this)
{
	CShape@ shape = this.getShape();
	shape.SetStatic(true);
	ShapeConsts@ consts = shape.getConsts();
	consts.mapCollisions = false;

	CSprite@ sprite = this.getSprite();
	sprite.SetZ(-50);

	this.chatBubbleOffset.x = 15; // doesnt work
	this.chatBubbleOffset.y = 90;

	this.addCommandID("use");
}

void onCommand(CBlob@ this, u8 cmd, CBitStream @params)
{
	if (cmd == this.getCommandID("use"))
	{
		CBlob@ blob = getBlobByNetworkID(params.read_netid());

		CRules@ rules = getRules();
		if (!getNet().isClient() || blob is null || hasMenus(rules) || Poker::hasLocalPokerSession(rules))
			return;

		CPlayer@ player = blob.getPlayer();
		// show bar menu
		ShowBarMenu( rules, this, blob );
	}
}

void ShowBarMenu(CRules@ this, CBlob@ bartender, CBlob@ player)
{
	UI::AddGroup(BAR_MENU, Vec2f(0.25f,0.25f), Vec2f(0.7f, 0.7f));
	UI::SetFont("gui");
	UI::Grid(1, 6);
	UI::Debug::AddButton("Greet", MenuGreet);
	UI::Debug::AddButton("Buy cigarettes (1c)", MenuCigarettes);
	UI::Debug::AddButton("Buy wine (1c)", MenuDrink);
	UI::Debug::AddButton("Buy beer (1c)", MenuDrink);
	UI::Debug::AddButton("Show drinking leaderboard", MenuLeaderboard);
	UI::Debug::AddButton("Goodbye", MenuBye);
	UI::SetSelector("none", Vec2f_zero);

	UI::getGroup(BAR_MENU).vars.set("player", @player);
	UI::getGroup(BAR_MENU).vars.set("bartender", @bartender);
}


void MenuGreet(CRules@ this, UI::Group@ group, UI::Control@ button)
{
	CBlob@ bartender;
	group.vars.get("bartender", @bartender);
	UI::Clear(group.name);
	if (bartender is null)
		return;
	bartender.Chat("Greetings to you");
}

void MenuPoker(CRules@ this, UI::Group@ group, UI::Control@ button)
{
	CBlob@ bartender;
	group.vars.get("bartender", @bartender);
	CBlob@ player;
	group.vars.get("player", @player);	
	UI::Clear(BAR_MENU);
	if (bartender is null || player is null)
		return;	

	bartender.Chat("Let's play!");
	Poker::StartSession( player );
	Poker::JoinPoker( bartender, player );
}

void MenuCigarettes(CRules@ this, UI::Group@ group, UI::Control@ button)
{
	CBlob@ bartender;
	group.vars.get("bartender", @bartender);
	CBlob@ player;
	group.vars.get("player", @player);		
	UI::Clear(BAR_MENU);
	if (bartender is null || player is null)
		return;	

	bartender.Chat("Enjoy");
	player.Chat("Thanks");
	CBitStream params;
	player.SendCommand(Soldier::Commands::CIVILIAN_CIGARETTE, params);			
}

void MenuDrink(CRules@ this, UI::Group@ group, UI::Control@ button)
{
	CBlob@ bartender;
	group.vars.get("bartender", @bartender);
	CBlob@ player;
	group.vars.get("player", @player);		
	UI::Clear(BAR_MENU);
	if (bartender is null || player is null)
		return;	

	bartender.Chat("Enjoy");
	player.Chat("Cheers!");
	CBitStream params;
	player.SendCommand(Soldier::Commands::CIVILIAN_DRINK, params);			
}

void MenuLeaderboard(CRules@ this, UI::Group@ group, UI::Control@ button)
{
	CBlob@ bartender;
	group.vars.get("bartender", @bartender);
	UI::Clear(BAR_MENU);
	if (bartender is null)
		return;	
	_leaderboardShowtime = getGameTime();		
}

void MenuBye(CRules@ this, UI::Group@ group, UI::Control@ button)
{
	CBlob@ bartender;
	group.vars.get("bartender", @bartender);
	UI::Clear(BAR_MENU);
	if (bartender is null)
		return;	
	bartender.Chat("See you later");
}

// SPRITE



void onInit(CSprite@ this)
{
	CBlob@ blob = this.getBlob();
	if (blob is null) return;

	bool vip = blob.hasTag("vip");

	for (u32 i = 1; i < (vip ? 5 : 6); i++)
	{
		CSpriteLayer@ block = this.addSpriteLayer("block" + i, "Sprites/bar.png", 8, 8, 0, 0);
		if (block !is null)
		{
			block.SetOffset(Vec2f(i * 8, 0));
			block.SetFrameIndex(i);
		}

		bool is_beer = (i == (vip ? 3 : 4));
		bool is_wine = (i == 1);
		if (is_beer || is_wine)
		{
			CSpriteLayer@ drink = this.addSpriteLayer("drink" + i, "Sprites/bar.png", 8, 8, 0, 0);
			if (drink !is null)
			{
				drink.SetOffset(Vec2f(i * 8 + (is_beer ? 4 : -4), -8));
				drink.SetFrameIndex(is_beer ? 6 : 7);
			}
		}
	}

	//bartender and anims
	CSpriteLayer@ bartender = this.addSpriteLayer("bartender", "Sprites/bar.png", 8, 8, 0, 0);
	if (bartender !is null)
	{
		bartender.SetOffset(Vec2f(16, -8));
		bartender.SetRelativeZ(-10);
		{
			Animation@ anim = bartender.addAnimation("mal", 0, false);
			int[] frames = {8};
			anim.AddFrames(frames);
		}
		{
			Animation@ anim = bartender.addAnimation("mal_leave", 4, true);
			int[] frames = {9, 10, 11};
			anim.AddFrames(frames);
		}
		{
			Animation@ anim = bartender.addAnimation("fem", 0, false);
			int[] frames = {12};
			anim.AddFrames(frames);
		}
		{
			Animation@ anim = bartender.addAnimation("fem_leave", 4, true);
			int[] frames = {13, 14, 15};
			anim.AddFrames(frames);
		}
	}

	CSpriteLayer@ neon = this.addSpriteLayer("neon", "Sprites/bar.png", 32, 8, 0, 0);
	if (neon !is null)
	{
		neon.SetOffset(Vec2f(16, -20));
		neon.SetRelativeZ(-20);
		{
			Animation@ anim = neon.addAnimation("default", 6, true);
			int[] frames =
			{
				6, 7, 8, 9,
				4, 5, 4, 5, 4, 5,
				6, 7, 6, 8, 6, 9, 6,
				4, 5, 4, 5
			};
			for (u32 i = 0; vip && i < frames.length; i++)
			{
				frames[i] += 6;
			}
			u32 vip_frame_offset = (vip ? 9 : 0);
			while (vip_frame_offset-- > 0)
			{
				frames.push_back(frames[0]); frames.removeAt(0);
			}
			anim.AddFrames(frames);
		}
	}
}

void onTick(CSprite@ this)
{
	CBlob@ b = this.getBlob();
	if (b is null) return;

	bool vip = b.hasTag("vip");

	//bartender

	//todo: leave/swap animations
	CSpriteLayer@ bartender = this.getSpriteLayer("bartender");
	if (bartender !is null)
	{
		u32 swaptime_seconds = 30;//(60 * 60 * 4);
		u8 gender = (Time() / swaptime_seconds + (vip ? 1 : 0)) % 2;
		string[] anims =
		{
			"mal",
			"fem"
		};
		if (gender < anims.length)
		{
			bartender.SetAnimation(anims[gender]);
		}
	}
}

void onRender(CSprite@ this)
{
	// render leaderboard
	if (getGameTime() - _leaderboardShowtime < LEADERBOARD_TIME)
	{
		CBlob@ blob = this.getBlob();

		if (blob.get_s16("in menu") > 0)
			return;
				
		uint player_count = 5;
		float per_line_offset = 1.75f;

		float camera_scaling = getCamera().targetDistance;
		float pixel_scale = camera_scaling * 2.0f;
		int score_cap = 10;

		Vec2f upperleft = Vec2f( getScreenWidth() / 4 - ((score_cap + 1) / 2) * 16.0f * pixel_scale,
		                        getScreenHeight() / 3 + (-player_count / 3) * 16.0f * pixel_scale);

		Vec2f totalsize = Vec2f(score_cap + 1, (1 + player_count) * per_line_offset) * 16.0f * pixel_scale;


		//background frame
		GUI::DrawRectangle(upperleft - Vec2f(18, 10), upperleft + totalsize + Vec2f(18, 10), Colours::BLACK);
		GUI::DrawRectangle(upperleft - Vec2f(17, 9), upperleft + totalsize + Vec2f(17, 9), Colours::PURPLE);
		GUI::DrawRectangle(upperleft - Vec2f(16, 8), upperleft + totalsize + Vec2f(16, 8), Colours::BLACK);

		float draw_scale = 0.5f;

		//"BOTTLES"
		GUI::DrawIcon(_score_spritesheet, 1,
		              Vec2f(64, 16), upperleft + Vec2f(totalsize.x / 2 - 64 * draw_scale, 0), draw_scale);


		//draw bottles
		for (uint i = 0; i < player_count; i++)
		{
			const uint score = player_count - i;
			Vec2f local_upperleft = upperleft + Vec2f(0, -8.0f + (i + 1) * per_line_offset * 16.0f * pixel_scale);

			for (uint j = 0; j < score; j++)
			{
				GUI::DrawText( "character name", local_upperleft + Vec2f(0.0f, 0.0f), color_white );
				GUI::DrawIcon(_score_spritesheet, 8, Vec2f(16, 16),
				              local_upperleft + Vec2f(4, 12) + (Vec2f(16, 0) * (j + 1) * pixel_scale),
				              draw_scale);
			}
		}
	}
}