shared class HoverMessage
{
	int index;
	int quantity;
	string name;
	uint ticker;
	Vec2f pos;
	uint ttl;
	uint fade_ratio;
	SColor color;

	HoverMessage() {} // required for handles to work

	HoverMessage(string _name, Vec2f position, int _quantity = 0, SColor _color = color_white, bool singularise = false, uint _ttl = 75, uint _fade_ratio = 2)
	{
		if (_quantity >= 0 && _quantity < 2 && singularise)
		{
			_name = this.Singularize(_name);
		}

		name = _name;
		quantity = _quantity;
		ticker = 0;
		ttl = _ttl;
		fade_ratio = _fade_ratio;
		color = _color;

		pos = position;
	}

	// draw the text
	void Draw(CBlob@ blob)
	{
		string m = this.Message();
		UpdatePos(blob, m);

		Vec2f drawpos = getDriver().getScreenPosFromWorldPos( pos );

		int top = drawpos.y - 1.5f * blob.getHeight() - 6.0f - index * 6.0f;
		int margin = 4;
		Vec2f dim;
		GUI::GetTextDimensions(m , dim);
		dim.x = Maths::Min(dim.x, 200.0f);
		dim.x += margin;
		dim.y += margin;
		dim.y /= 3.8f;
		drawpos.x = drawpos.x - dim.x / 2;
		drawpos.y = top - 1 * dim.y;

		GUI::DrawText(m, drawpos, getColor());
	}

	// get message into a nice, friendly format
	string Message()
	{
		if (quantity == 0)
		{
			return name;
		}
		return "+" + quantity + " " + name;
	}

	// see if this message is expired, or should be removed from GUI
	bool isExpired()
	{
		ticker = ticker + 1;
		return ticker > ttl;
	}

	// get the active color of the message. decrease proportionally by the fadeout ratio
	private SColor getColor()
	{
		uint alpha = Maths::Max(0, 255 - (ticker * fade_ratio));
		SColor color2 = SColor(alpha, color.getRed(), color.getGreen(), color.getBlue());
		return color2;
	}

	// get the position of the message. Store it to the object if no pos is already set. This allows us to do the
	// hovering above where it was picked effect. Finally, slowly make it rise by decreasing by a multiple of the ticker
	private void UpdatePos(CBlob@ blob, string m)
	{
		/*if (pos.getLengthSquared() == 0.0)
		{
			Vec2f pos2d = blob.getScreenPos();
			int top = pos2d.y - 2.5f * blob.getHeight() - 10.0f - index * 10.0f;
			int margin = 4;
			Vec2f dim;
			GUI::GetTextDimensions(m , dim);
			dim.x = Maths::Min(dim.x, 200.0f);
			dim.x += margin;
			dim.y += margin;
			dim.y /= 3.8f;

			pos.x = pos2d.x - dim.x / 2;
			pos.y = top - 2 * dim.y;
		}*/

		pos.y = pos.y - (ticker / (40));
	}

	// Singularize, or de-pluralize, a string
	private string Singularize(string str)
	{
		uint len = str.length();
		string lastChar = str.substr(len - 1);

		if (lastChar == "s")
		{
			str = str.substr(0, len - 1);
		}

		return str;
	}
};

HoverMessage[]@ getMessages(CBlob@ this)
{
	if (!this.exists("messages"))
	{
		HoverMessage[] messages;
		this.set("messages", messages);
	}
	HoverMessage[]@ p_messages;
	this.get("messages", @p_messages);
	return p_messages;
}

bool isSameMessageAlready(HoverMessage[]@ messages, const string &in label)
{
	for (uint i = 0; i < messages.length; i++)
	{
		HoverMessage @message = messages[i];
		if (message.name == label && message.ticker < message.ttl / 2.0f)
			return true;
	}
	return false;
}

void AddMessage(CBlob@ this, const string &in label, const int quantity = 0)
{
	if (this is null)
		return;
	HoverMessage[]@ messages = getMessages(this);
	if (!isSameMessageAlready(messages, label))
	{
		HoverMessage m(label, this.getPosition(), quantity);
		messages.push_back(m);
		messages[messages.length - 1].index = messages.length - 1;
	}
}

void ClearMessages(CBlob@ this)
{
	if (this is null)
        return;

	HoverMessage[]@ msg = getMessages(this);
	if (msg !is null)
		msg.clear();
}

void AddScore(CBlob@ this, const int quantity)
{
	if (this is null)
		return;
	HoverMessage[]@ messages = getMessages(this);
	HoverMessage m("", this.getPosition(), quantity);
	messages.push_back(m);
	messages[messages.length - 1].index = messages.length - 1;
}