#include "SoldierCommon.as"
#include "ExplosionParticles.as"

void onInit(CSprite@ this)
{
	this.ReloadSprite( "actor_civ_fem.png" ); // fixes engine bug (consts.filename = filename not set in ReloadSprite overload)
	this.ReloadSprite("actor_civ_fem.png", 24, 24, Soldier::getTeamColorForSprite(this.getBlob()), 0 );

	{
		Animation@ anim = this.addAnimation("stand", 0, false);
		anim.AddFrame(0);
	}
	{
		Animation@ anim = this.addAnimation("run", 3, true);
		int[] frames = {2, 3, 4, 7, 8, 9};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("jump up", 3, false);
		int[] frames = {15, 16, 17};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("jump down", 3, false);
		int[] frames = {17, 18, 19};
		anim.AddFrames(frames);
	}

	{
		Animation@ anim = this.addAnimation("crouch", 2, false);
		int[] frames = {10, 11, 12};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("stand up", 1, false);
		int[] frames = {12, 13, 10};
		anim.AddFrames(frames);
	}

	{
		Animation@ anim = this.addAnimation("slide start", 3, false);
		int[] frames = {13, 5, 6};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("slide", 3, true);
		int[] frames = {5, 6};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("die", 2, false);
		int[] frames = {20, 21, 22, 23};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("ground", 0, false);
		anim.AddFrame(14);
	}
	{
		Animation@ anim = this.addAnimation("fall up", 0, false);
		anim.AddFrame(22);
	}
	{
		Animation@ anim = this.addAnimation("fall down", 0, false);
		anim.AddFrame(24);
	}
	{
		Animation@ anim = this.addAnimation("crawl", 6, true);
		int[] frames = {14};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("bite", 3, false);
		int[] frames = {14};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("agony", 5, false);
		int[] frames = {14};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("flip", 4, false);
		int[] frames = {1};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("ladder", 0, false);
		int[] frames = {20, 21};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("smoke_drag", 4, true);
		int[] frames = {24, 22, 22, 22, 22, 22, 24};
		anim.AddFrames(frames);
	}
	{
		Animation@ anim = this.addAnimation("smoke_talk", 3, true);
		int[] frames = {23, 23, 24, 24, 23, 23, 23, 23, 24, 24, 24, 23, 23, 24};
		anim.AddFrames(frames);
	}
}

void onTick(CSprite@ this)
{
	CBlob@ blob = this.getBlob();
	Soldier::Data@ data = Soldier::getData(blob);

	if (data.dead)
	{
		return;
	}

	//find out if has cigs here
	if (data.attached)
	{
		//mask other anims and just crouch for now
		data.specialAnim = true;
		this.SetAnimation("crouch");
		return;
	}

	//todo: find out if we have cigs here
	const bool still = data.vel.getLengthSquared() < 0.5f;
	bool smoking = false;
	bool drinking = false;
	if (still && blob.hasTag("smoking")) {
		smoking = true;
	}
	if (still && blob.hasTag("drinking")) {
		drinking = true;
	}

	data.specialAnim = smoking || drinking;

	if (data.specialAnim)
	{
		if(this.isAnimationEnded())
		{
			if (drinking)
			{
				if(this.isAnimation("smoke_drag"))
				{
					this.PlayRandomSound("Gulp");
					blob.Untag("drinking");
				}

				this.SetAnimation("smoke_drag");
			}
			else if (smoking)
			{
				if(this.isAnimation("smoke_drag"))
				{
					f32 x = data.facingLeft ? -2 : 2;
					Particles::MicroDusts(data.pos + Vec2f(x, -4), 3, data.vel + Vec2f(x, 0) * 0.2f, 1.0f);
					this.PlaySound("SmokeCigarette");
				}

				Random _r(Time());
				string[] possible = {"drag", "talk"};
				this.SetAnimation("smoke_"+possible[_r.NextRanged(possible.length)]);				
			}
		}
	}
}
