#include "SoldierCommon.as"
#include "SoldierCrosshair.as"

float MAX_THROW = 8.0f;
float USE_DISTANCE = 30.0f;

int _useLock = 0;

void onInit(CBlob@ this)
{
	//this.getCurrentScript().runFlags |= Script::tick_ininventory | Script::tick_not_ininventory;
}

void onTick(CBlob@ this)
{
	Soldier::Data@ data = Soldier::getData(this);

	if (data.dead)
		return;

	// get out of truck
	if (data.attached && getNet().isServer())
	{
		CBlob@ truck = this.getAttachments().getAttachedBlob("PASSENGER");
		if (this.isKeyPressed(key_action2) && truck !is null
			&& !truck.hasTag("riding away")
			&& truck.get_u8("leave_secs") > 3
			)
		{
			this.server_DetachFromAll();
		}
	}

	// ball

	CBlob@ ball = this.getCarriedBlob();
	if (ball !is null)
	{
		if (data.local)
		{
			if (this.isKeyPressed(key_action2))
			{
				f32 drop_speed = 1.5f;
				Vec2f vel = data.vel * 0.8f + Vec2f(data.facingLeft ? -0.5f : 0.5f, -1.0f) * drop_speed;
				Throw(this, ball, vel);
			}
			else if (!data.crosshair && this.isKeyPressed(key_action1))
			{
				Soldier::StartCrosshair(this, data);
			}
			else if (data.crosshair)
			{
				if (this.isKeyJustPressed(key_action1) || (data.crosshairTime >= data.crosshairMinTime && this.isKeyJustReleased(key_action1)))
				{
					Vec2f vel = data.crosshairOffset * 0.05f;
					f32 len = vel.Normalize();
					vel *= Maths::Min(len, MAX_THROW);
					vel += data.vel;
					Throw(this, ball, vel);
				}
			}
		}
	}
	else
	{
		Soldier::EndCrosshair(this, data);
	}

	Soldier::TickCrosshair(this, data);

	if (data.local && _useLock == 0 && ball is null)
	{
		// use

		if (this.isKeyJustPressed(key_action1))
		{
			CBlob@[] blobsInRadius;
			if (this.getMap().getBlobsInRadius(this.getPosition(), USE_DISTANCE, @blobsInRadius))
			{
				for (uint i = 0; i < blobsInRadius.length; i++)
				{
					CBlob @b = blobsInRadius[i];
					if (b !is this && this.getDistanceTo(b) < b.getRadius() && b.hasCommandID("use"))
					{
						Use(this, b);
						break;
					}
				}
			}
		}

		// smoke

		if (this.isKeyJustPressed(key_action2))
		{
			////CBitStream params;
			//this.SendCommand(Soldier::Commands::CIVILIAN_CIGARETTE, params);			
		}	
	}

	// use lock
	if (data.local)
	{
		if (getRules().get_s16("in menu") == 0){
			_useLock = 0;
		}
		else{
			_useLock = 1;
		}	
	}
}


void Throw(CBlob@ this, CBlob@ ball, Vec2f vel)
{
	CBitStream params;
	params.write_netid(ball.getNetworkID());
	params.write_Vec2f(ball.getPosition());
	params.write_Vec2f(vel);
	this.SendCommand(Soldier::Commands::THROWBALL, params);
}

//stealing the ball
void onCollision(CBlob@ this, CBlob@ blob, bool solid, Vec2f normal, Vec2f point1)
{
	if (!getNet().isServer())
	{
		return;
	}

	if (blob is null || !blob.hasTag("player"))
	{
		return;
	}

	Soldier::Data@ my_data = Soldier::getData(this);
	Soldier::Data@ their_data = Soldier::getData(blob);
	if (my_data is null || their_data is null)
	{
		return;
	}

	CBlob@ ball = this.getCarriedBlob();
	if (ball is null)
	{
		return;
	}

	if (my_data.vellen > their_data.vellen)
	{
		ball.server_DetachFromAll();
		ball.server_AttachTo(this, 0);
	}

}

void onCommand(CBlob@ this, u8 cmd, CBitStream @params)
{
	Soldier::Data@ data = Soldier::getData(this);

	if (cmd == Soldier::Commands::THROWBALL)
	{
		CBlob@ ball = getBlobByNetworkID(params.read_netid());
		Vec2f pos = params.read_Vec2f();
		Vec2f velocity = params.read_Vec2f();

		if (getNet().isServer())
		{
			ball.server_DetachFromAll();
		}

		ball.setPosition(pos);
		ball.setVelocity(velocity);
	}
	else if (cmd == Soldier::Commands::CIVILIAN_CIGARETTE)
	{
		this.Tag("smoking");
		this.getSprite().PlaySound("LightCigarette");
	}
	else if (cmd == Soldier::Commands::CIVILIAN_DRINK)
	{
		this.Tag("drinking");
		this.getSprite().PlayRandomSound("Gulp");
	}	
}

void Use(CBlob@ this, CBlob@ other)
{
	CBitStream params;
	params.write_netid(this.getNetworkID());
	other.SendCommand(other.getCommandID("use"), params);
}
