#include "SoldierCommon.as"
#include "SoldierPlace.as"
#include "SoldierCrosshair.as"
#include "HoverMessage.as"

const int PATRIOTS = 3;
const float SHOOT_VEL = 1.5f;
const float SHOOT_OFFSET = -8.0f;

enum EngineerState
{
	ES_IDLE = 0,
	ES_AIMING,
	ES_FIRED,
};

void onInit(CBlob@ this)
{
	Soldier::Data@ data = Soldier::getData(this);
	data.walkSpeedModifier = 1.0f;

	data.grenades = data.initialGrenades = PATRIOTS;
	data.crosshairMaxDist = 9999.0f;
}

void onTick(CBlob@ this)
{
	Soldier::Data@ data = Soldier::getData(this);

	if (data.dead || data.stunned || getRules().isWarmup())
	{
		data.engineerState = ES_IDLE; //prevent locking up when die while firing
		data.missileId = 0;
		data.lockCrouch = 0;
		return;
	}

	if (!data.crosshair && !data.fire2 && data.missileId == 0 && this.isKeyJustPressed(key_action1))
	{
		if (data.grenades > 0)
		{
			Soldier::StartCrosshair(this, data, true, 0.0f);
			data.engineerState = ES_AIMING;
			data.crosshairMaxDist = 30.0f;
		}
		else if (data.local)
		{
			data.sprite.PlaySound("NoAmmo");
			AddMessage(this, "no missiles");
		}
	}
	else if (data.crosshair || data.missileId > 0)
	{
		if (this.isKeyPressed(key_action2) || (data.missileId > 0 && getBlobByNetworkID(data.missileId) is null)
		        || (data.missileId > 0 && this.isKeyJustPressed(key_action1) && data.engineerState == ES_FIRED))
		{
			data.engineerState = ES_IDLE;
			//data.lockCrouch = 0;
			data.sprite.PlaySound("GrenadePut");
			Soldier::EndCrosshair(this, data);
			data.missileId = 0;
		}
		else if (data.engineerState == ES_AIMING)
		{
			if (this.isKeyJustPressed(key_action1) || (data.crosshairTime >= data.crosshairMinTime && this.isKeyJustReleased(key_action1)))
			{
				if (data.local)
				{
					CBitStream params;
					params.write_Vec2f(data.crosshairOffset);
					this.SendCommand(Soldier::Commands::ENGIE_SHOOT, params);
				}
				data.engineerState = ES_FIRED;
			}
		}
	}

	Soldier::TickCrosshair(this, data);
}

CBlob@ ShootPatriot(CBlob@ this, Soldier::Data@ data)
{
	CMap@ map = getMap();
	Vec2f pos = data.pos + Vec2f(0.0f, SHOOT_OFFSET);
	Vec2f vec = data.crosshairOffset;
	vec.Normalize();
	data.crosshairOffset = (pos - data.pos) + vec * 20.0f;
	data.crosshairMaxDist = 9999.0f;
	ShakeScreen(vec * -30.0f, 30, data.pos);

	data.sprite.PlaySound("PatriotLaunch");
	if (!getRules().get_bool("infinite grenades") && !sv_test)
	{
		data.grenades--;
	}

	if(!getNet().isServer())
	{
		return null;
	}

	CBlob @blob = server_CreateBlob("patriot", this.getTeamNum(), pos);
	if (blob !is null)
	{
		blob.setVelocity(vec * SHOOT_VEL);
		blob.set_netid("owner", this.getNetworkID());
		blob.SetDamageOwnerPlayer(this.getPlayer());
		blob.SetFacingLeft(this.isFacingLeft());
		data.missileId = blob.getNetworkID();

		CBitStream params;
		params.write_netid(data.missileId);
		this.SendCommand(Soldier::Commands::ENGIE_CONTROL, params);
	}
	return blob;
}

void onCommand(CBlob@ this, u8 cmd, CBitStream @params)
{
	if (cmd == Soldier::Commands::ENGIE_SHOOT)
	{
		Soldier::Data@ data = Soldier::getData(this);
		data.crosshairOffset = params.read_Vec2f();
		ShootPatriot(this, data);
	}
	else if (cmd == Soldier::Commands::ENGIE_CONTROL)
	{
		Soldier::Data@ data = Soldier::getData(this);
		data.missileId = params.read_netid();
	}
}
