#define SERVER_ONLY

#include "SimpleStates.as"
#include "SoldierCommon.as"
#include "SimpleCommonStates.as"

void onInit( CBrain@ this )
{
	CBlob@ blob = this.getBlob();
	SimpleBrain::States@ states = SimpleBrain::getStates( blob );

	states.available.push_back( SimpleBrain::State( "move", Prioritize_Move, Do_Move ) );
	states.available.push_back( SimpleBrain::State( "enemy visible", Prioritize_EnemyVisible, Do_EnemyVisible ) );
	states.available.push_back( SimpleBrain::State( "escape grenade", Prioritize_EscapeGrenade, Do_EscapeGrenade ) );
	states.available.push_back( SimpleBrain::State( "aim", Prioritize_Aim, Do_Aim ) );
	SimpleBrain::SetupMoveVars( states );
}

// MOVE

void Prioritize_Move( CBlob@ blob, SimpleBrain::State@ state )
{
	f32 p = 0.5f;
	state.priority = p;
}

void Do_Move( CBlob@ blob, SimpleBrain::State@ state )
{
	SimpleBrain::GoombaMovement( blob, state );
}

// ENEMY_VISIBLE

void Prioritize_EnemyVisible( CBlob@ blob, SimpleBrain::State@ state )
{
	CBlob@ enemy = SimpleBrain::getVisibleEnemy( blob, SimpleBrain::VISIBLE_DISTANCE );	

	f32 p = 0.0f;
	if (enemy !is null){
		p += 0.6f;
	}	

	state.priority = p;
}

void Do_EnemyVisible( CBlob@ blob, SimpleBrain::State@ state )
{
	CBrain@ brain = blob.getBrain();
	CBlob @target = brain.getTarget();

	// find enemy target if no target
	if (target is null){
		@target = SimpleBrain::getVisibleEnemy( blob, SimpleBrain::VISIBLE_DISTANCE );
		brain.SetTarget( target );
	}	
	else
	{
		blob.setKeyPressed( key_crouch, true );
		// crate
		if (XORRandom(125) == 0){
			blob.setKeyPressed( key_action2, true );
		}		
	}	
}

// ESCAPE_GRENADE

void Prioritize_EscapeGrenade( CBlob@ blob, SimpleBrain::State@ state )
{
	f32 p = 0.0f;
	Vec2f pos = blob.getPosition();
	CBlob@ grenade = SimpleBrain::getVisibleBlobWithTag( pos, "explosive", SimpleBrain::EXPLOSIVE_DISTANCE );
	if (grenade !is null){
		p += 1.0f;
	}	

	state.priority = p;
}

void Do_EscapeGrenade( CBlob@ blob, SimpleBrain::State@ state )
{
	SimpleBrain::EscapeExplosive( blob, state );	
}

// AIM

void Prioritize_Aim( CBlob@ blob, SimpleBrain::State@ state )
{
	Vec2f pos = blob.getPosition();	
	Soldier::Data@ data = Soldier::getData( blob );
	CBlob@ nearby = SimpleBrain::getNearbyEnemy( blob, SimpleBrain::VISIBLE_DISTANCE );		

	f32 p = 0.0f;

	if (data.crosshair){ // dont do anything else while cooking nade
		p += 10.0f;
	}

	if (nearby !is null)
	{	
		const f32 goodDist = 15.0f;		
		if (SimpleBrain::isStraightVisible( blob.getPosition(), nearby.getPosition() )){
			p += 0.6f;
		}							
	}

	state.priority = p;	
}

void Do_Aim( CBlob@ blob, SimpleBrain::State@ state )
{
	CBrain@ brain = blob.getBrain();
	CBlob @target = brain.getTarget();
	Vec2f pos = blob.getPosition();
			
	// find enemy target if no target
	if (target is null){
		@target = SimpleBrain::getNearbyEnemy( blob, SimpleBrain::VISIBLE_DISTANCE );
		brain.SetTarget( target );
	}	
	else
	{
		Soldier::Data@ data;
		blob.get( "data", @data );
		if (data.crosshair)
		{
			blob.setKeyPressed( key_action1, false );
		}

		if (XORRandom(50) == 0){
			blob.setKeyPressed( key_action1, true );
		}
	}
}