#include "SoldierCommon.as"
#include "SoldierPlace.as"
#include "Sparks.as"
#include "GameColours.as"
#include "SoldierRevive.as"

const int SUPPLIES = 3;
const f32 HEAL_RADIUS = 16.0f;
const int HEAL_DELAY = 45;

void onInit(CBlob@ this)
{
	Soldier::Data@ data = Soldier::getData(this);
	data.walkSpeedModifier = 1.2f;

	InitPlaceOrDelete(this, "supply", SUPPLIES);
}

void onTick(CBlob@ this)
{
	Soldier::Data@ data = Soldier::getData(this);

	const bool matchrunning = getRules().isMatchRunning();

	//get healed by others who're alive
	/*if (data.dead && matchrunning)
	{
		healSelfIfBlobsNearby(this);
	}*/

	if (data.dead || data.stunned || !matchrunning)
	{
		data.shield = false;
		return;
	}

	// shield

	if (data.fire && data.stunTime <= 0)
	{
		data.shield = true;
		data.allowCrouch = false;
		this.Tag("collide with nades");
	}
	else
	{
		data.shield = false;
		data.allowCrouch = true;
		this.Untag("collide with nades");
	}

	if (data.local && this.isKeyJustPressed(key_action2))
	{
		if (data.supplyTime - getGameTime() < 0){
			Supply(this, data);
		}
		else {
			data.sprite.PlaySound("NoAmmo");
		}
	}

	// heal

	//healBlobsNearby(this);
}

void Supply(CBlob@ this, Soldier::Data@ data)
{
	CBitStream params;
	params.write_netid(this.getNetworkID());
	params.write_Vec2f(this.getPosition());
	params.write_Vec2f(this.getVelocity());
	this.SendCommand(Soldier::Commands::MEDIC_SUPPLY, params);
}


void onCommand(CBlob@ this, u8 cmd, CBitStream @params)
{
	Soldier::Data@ data = Soldier::getData(this);

	if (cmd == Soldier::Commands::MEDIC_SUPPLY)
	{
		CBlob@ caller = getBlobByNetworkID(params.read_netid());
		Vec2f pos = params.read_Vec2f();
		Vec2f vel = params.read_Vec2f();

		if (getNet().isServer())
		{
			CBlob@ supply = PlaceBlobNoInit("supply", data.pos, this, this.getTeamNum());
			if (supply !is null)
			{
				supply.set_u8("supply type", 2);
				supply.setVelocity( vel*1.5f );
				supply.Init();
			}
		}

		data.supplyTime = getGameTime() + Soldier::medicSupplyInterval;
		this.getSprite().PlaySound("GiveOut");
	}
}

void healBlobsNearby(CBlob@ this)
{
	CBlob@[] blobsInRadius;
	if (!this.getMap().getBlobsInRadius(this.getPosition(), HEAL_RADIUS, @blobsInRadius))
		return;

	const u8 team = this.getTeamNum();
	for (uint i = 0; i < blobsInRadius.length; i++)
	{
		CBlob @b = blobsInRadius[i];
		if (b.getTeamNum() == team && b.getHealth() < b.getInitialHealth() && b.getName() == "soldier")
		{
			Soldier::Data@ data = Soldier::getData(b);
			data.healTime += 2;

			if (data.healTime > HEAL_DELAY)
			{
				printf("heal");

				//todo: maybe 1up sound here? that or some sort of screen flash

				data.healTime = 0;
				string sound = data.dead ? "MedkitRevive" : "MedkitHeal";
				this.getSprite().PlayRandomSound(sound, 1.0f, data.pitch);
				if (getNet().isServer())
				{
					Revive(b);
				}
			}

			break;
		}
	}
}

void healSelfIfBlobsNearby(CBlob@ this)
{
	CBlob@[] blobsInRadius;
	if (!this.getMap().getBlobsInRadius(this.getPosition(), HEAL_RADIUS, @blobsInRadius))
		return;

	Soldier::Data@ mydata = Soldier::getData(this);

	const u8 team = this.getTeamNum();
	for (uint i = 0; i < blobsInRadius.length; i++)
	{
		CBlob @b = blobsInRadius[i];
		if (b.getTeamNum() == team && b.getName() == "soldier")
		{
			Soldier::Data@ data = Soldier::getData(b);
			if(data.dead)
				continue;

			mydata.healTime += 2;

			//stay still!
			if (mydata.onGround)
			{
				mydata.stunTime = 30;
			}

			if (mydata.healTime > HEAL_DELAY)
			{
				printf("heal self");

				//todo: maybe 1up sound here? that or some sort of screen flash

				mydata.healTime = 0;
				this.getSprite().PlayRandomSound("MedkitRevive", 1.0f, data.pitch);
				if (getNet().isServer())
				{
					Revive(this);
				}
			}

			break;
		}
	}
}


f32 onHit(CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitterBlob, u8 customData)
{
	Soldier::Data@ data = Soldier::getData(this);
	if (data.shield)
	{
		if ((velocity.x < 0.0f && !data.facingLeft) || (velocity.x > 0.0f && data.facingLeft))
		{
			this.getSprite().PlayRandomSound("ShieldImpact");
			Particles::Sparks(worldPoint, 4, velocity.getLength() * 0.4f, SColor(Colours::YELLOW));

			// push back
			//printf("velocity " + velocity.x + " da " + damage);
			data.vel += velocity;
			data.vel.y = -Maths::Abs(data.vel.y) - 0.75f * damage;
			this.setVelocity(data.vel);

			// stun
			if (damage > 1.0f)
			{
				data.stunTime = 120;
			}

			damage = 0.0f;
		}
	}
	return damage;
}
