#include "HoverMessage.as"
#include "GameColours.as"
#include "BackendCommon.as"
#include "LobbyCommon.as"
#include "LobbyStatsCommon.as"
#include "Timers.as"
//so we can hide when rendering menus..
#include "UI.as"


//TODO: paged rendering;
//		- server
// 		- players
//		- games played
// 		- countdown (priority)

void onInit(CBlob@ this)
{
	CShape@ shape = this.getShape();
	shape.SetStatic(true);
	ShapeConsts@ consts = shape.getConsts();
	consts.mapCollisions = false;

	CSprite@ sprite = this.getSprite();
	sprite.SetZ(1000);
}

void onTick(CBlob@ this)
{
	if (getRules() is null || !getNet().isServer())
		return;

	Lobby::Server[]@ servers = Lobby::getServers();
	if (servers !is null && servers.length > 0)
	{
		LobbyStats@ stats = getStats();

		//collect lines here
		string[] lines;

		//server count
		{
			//count number initialised
			u32 amount = Maths::Min(servers.length, 9); //force to 9 maximum - we cant display more!
			u32 connectable = 0;
			for (uint i = 0; i < servers.length; i++)
			{
				if (servers[i].connectable = true)
				{
					connectable++;
				}
			}

			lines.push_back(connectable + "/" + amount + " servers");
		}

		//time between game
		{
			string line = "";

			u32 seconds = stats.secondsBetweenGames();
			if (seconds == 0 || seconds > 600)
				line = "unknown";
			else if (seconds < 60)
				line = formatInt(seconds, "", 3) + "s";
			else
				line = formatInt(seconds / 60, "", 3) + "m";

			line += " wait";

			lines.push_back(line);
		}

		//player count
		lines.push_back("now:      ");
		{
			u32 players = getRules().get_u32("total_players");

			lines.push_back(formatInt(players, "", 3) + " player" + (players == 1 ? "" : "s"));
		}

		//daily stats
		lines.push_back("today:     ");
		{
			u32 players = stats.playersToday();
			lines.push_back(formatInt(players, "", 3) + " player" + (players == 1 ? "" : "s"));
		}

		lines.push_back("today:     ");
		{
			u32 games = stats.gamesToday;
			lines.push_back(formatInt(games, "", 3) + " game" + (games == 1 ? "" : "s"));
		}


		this.set_string("lines", join(lines, "||"));
		this.Sync("lines", true);

		//tag for rendering
		this.Tag("_init");
		this.Sync("_init", true);


		//print each 5 min for analytics' sake
		if((getGameTime() % 30 == 0) && Time() % (5 * 60) == 0 )
		{
			print("stats: " + "\n\t" +
				"players now " + getRules().get_u32("total_players") + "\n\t" +
				"players today " + stats.playersToday() + "\n\t" +
				"classes " + stats.classCount(0) + " " + stats.classCount(1) + " " + stats.classCount(2) + " " + stats.classCount(3) + " " + stats.classCount(4) + "\n\t" +
				"teams " + stats.teamCount(0) + " " + stats.teamCount(1) + "\n\t" +
				"games today " + stats.gamesToday + "\n\t" +
				"games total " + stats.gamesTotal() + "\n\t" +
				"seconds between games " + stats.secondsBetweenGames() + "\n\t" +
				lines[0] //server count
			);
		}

	}
}

void onInit(CSprite@ this)
{
	if (!GUI::isFontLoaded("computer screen"))
	{
		GUI::LoadFont("computer screen", "GUI/Fonts/computer/F25_Bank_Printer_Bold.ttf", 8, false);
	}
}

void onRender(CSprite@ this)
{
	if (UI::hasAnyContent())
		return;

	CBlob@ blob = this.getBlob();
	if (blob is null) return;

	Vec2f screenPos = blob.getScreenPos() + Vec2f(-2, -2);

	GUI::SetFont("computer screen");

	if (!blob.hasTag("_init"))
	{
		GUI::DrawTextCentered("CONNECTING", screenPos, Colours::GREEN);
		return;
	}

	const u8 state = getRules().getCurrentState();
	if (state == WARMUP)
	{
		s32 secs_left = Game::getTimerSecondsLeft("launch");

		f32 wiggle = Maths::Sin(getGameTime() * 0.2f);

		if (secs_left > 3)
		{
			Vec2f offset(2.0f * wiggle, -5);
			GUI::DrawTextCentered("GAME", screenPos + offset, Colours::GREEN);
			offset.y *= -1;
			GUI::DrawTextCentered("STARTING", screenPos + offset, Colours::GREEN);
		}
		else
			GUI::DrawTextCentered(formatInt(secs_left, "", 1), screenPos + Vec2f(0, 2.0f * wiggle), Colours::GREEN);

		return;
	}

	string[] lines = blob.get_string("lines").split("||");

	if (lines.length <= 1)
		return;

	int i = (Time() / 5);

	int current_line = (i % (lines.length / 2)) * 2;

	Vec2f offset(0, -5);
	GUI::DrawTextCentered(lines[current_line], screenPos + offset, Colours::GREEN);

	offset.y *= -1;
	GUI::DrawTextCentered(lines[(current_line + 1) % lines.length], screenPos + offset, Colours::GREEN);
}
