namespace Campaign
{
	const string SYNC_CMD = "campaign data";
	const string sync_ids_name = SYNC_CMD + "_bs";
	CBitStream _syncBs;

	shared class Data
	{
		string[] battles;
		string[] allBattles;
		int[] team;
		int battleIndex;
		int battlesCount;

		Data(){
			battleIndex = 0;
		}
	};

	Data@ InitCampaign( CRules@ this)
	{
		Data data;
		this.set("campaign", @data );
		this.addCommandID(SYNC_CMD);
		_syncBs.Clear();
		return getCampaign( this );
	}

	Data@ getCampaign( CRules@ this )
	{
		Data@ data;
		this.get( "campaign", @data );
		return data;
	}

	string getCurrentBattle( CRules@ this )
	{
		Data@ data = getCampaign( this );
		return data.battles[data.battleIndex];
	}

	void Reset( Data@ data )
	{
		// randomize battles

		Random _r(Time());
		string[] temp = data.allBattles;
		data.battles.clear();
		int index;
		while (data.battles.length < data.battlesCount)
		{
			index = _r.NextRanged(temp.length);
			data.battles.push_back(temp[index]);
			temp.removeAt(index);
		}

		// center battle
		data.battleIndex = 0;
		data.team.clear();
		for (uint i=0; i < data.battles.length; i++){
			data.team.push_back(-1);
		}
		_syncBs.Clear();
	}

	void Sync(CRules@ this, Data@ data)
	{
		if (getNet().isServer())
		{
			_syncBs.Clear();

			_syncBs.write_u16( data.battles.length );
			for (uint i = 0; i < data.battles.length; i++) {
				_syncBs.write_string( data.battles[i] );
			}

			_syncBs.write_u16( data.team.length );
			for (uint i = 0; i < data.team.length; i++) {
				_syncBs.write_s16( data.team[i] );
			}

			_syncBs.write_u16( data.battleIndex );

			this.set_CBitStream(sync_ids_name, _syncBs);
			this.Sync(sync_ids_name, true);
		}
		else
		{
			this.get_CBitStream(sync_ids_name, _syncBs);
			if (_syncBs.getBytesUsed() > 0)
			{
				_syncBs.ResetBitIndex();

				data.battles.clear();
				const u16 battles = _syncBs.read_u16();
				for (uint i = 0; i < battles; i++) {
					data.battles.push_back( _syncBs.read_string() );
				}

				data.team.clear();
				const u16 teams = _syncBs.read_u16();
				for (uint i = 0; i < teams; i++) {
					data.team.push_back( _syncBs.read_s16() );
				}

				data.battleIndex = _syncBs.read_u16();
			}
			else
			{
				data.battles.clear();
				data.team.clear();
				data.battleIndex = 0;
			}
		}
	}

	void SetWinMsg(CRules@ this, const string &in msg)
	{
		this.set_string("win msg", msg);
		this.Sync("win msg", true);
	}

	int getTeamScore( Data@ data, const int team )
	{
		int score = 0;
		for (uint i=0; i < data.battles.length; i++){
			if (data.team[i] == team){
				score++;
			}
		}
		return score;
	}

	bool isSeriesEnded( Data@ data )
	{
		const int team1score = getTeamScore(data, 0);
		const int team2score = getTeamScore(data, 1);
		const int scoreThreshold = Maths::Floor(data.battles.length/2)+1;

		// not initialized?
		if (getPlayersCount() == 0 || data.battles.length == 0){
			return false;
		}
//printf("getRules().getTeamWon() " + getRules().getTeamWon() + " data.battleIndex " + data.battleIndex + "/" + data.battles.length + " scoreThreshold " + scoreThreshold );
		return (getRules().getTeamWon() == -10 || data.battleIndex > data.battles.length - 1
				|| team1score >= scoreThreshold || team2score >= scoreThreshold
				);
	}

	// Portraits


	shared class Portrait
	{
		string name;
		int team, icon;
		string[] fields;
		f32 showTimer;
		CPlayer@ player;
		string chat;
		f32 chatTimer;
	};
}
