#include "CampaignCommon.as"
#include "GameColours.as"
#include "Timers.as"
#include "RulesCommon.as"
#include "RadioCharacters.as"
#include "ClassesCommon.as"
#include "PlayerStatsCommon.as"

string _locations_file = "Sprites/UI/location_icons.png";

string[] _positiveChats =
{
	"Go team!",
	"Together!",
	"Hurray!",
	"Yeah!",
	"We'll get em!"
};
string[] _negativeChats =
{
	"Wanker!",
	"Baddie!",
	"Stinker!",
	"Pigface!",
	"Poophead!",
	"Fartbutt!",
	"Idiot!"
};

const f32 SCALE = 1.0f;
Vec2f ICON_SIZE(SCALE * 32, SCALE * 32);
Vec2f ICON_DISPLAY_SIZE(ICON_SIZE.x + SCALE * 8, ICON_SIZE.y + SCALE * 8);

Campaign::Portrait[] _portraits;

void onInit(CRules@ this)
{
	this.addCommandID("say on scores");

	AddIconToken("$location_none$", _locations_file, Vec2f(32, 32), 0);
	AddIconToken("$location_trenches$", _locations_file, Vec2f(32, 32), 1);
	AddIconToken("$location_forest$", _locations_file, Vec2f(32, 32), 2);
	AddIconToken("$location_desert$", _locations_file, Vec2f(32, 32), 3);
	AddIconToken("$location_swamp$", _locations_file, Vec2f(32, 32), 4);
	AddIconToken("$location_village$", _locations_file, Vec2f(32, 32), 5);
	AddIconToken("$location_city$", _locations_file, Vec2f(32, 32), 6);
	AddIconToken("$location_mountain$", _locations_file, Vec2f(32, 32), 7);
	AddIconToken("$location_test$", _locations_file, Vec2f(32, 32), 0);
	AddIconToken("$location_done$", _locations_file, Vec2f(32, 32), 8);
}


void onStateChange(CRules@ this, const u8 oldState)
{
	const u8 state = this.getCurrentState();
	if (state == INTERMISSION)
	{
		printf("RESET STATE VIEEW");
		_portraits.clear();
	}
}

void onTick(CRules@ this)
{
	CControls@ controls = getControls();
	if (controls is null || !getNet().isClient())
		return;

	if (this.isIntermission())
	{
		UpdatePlayerPortraits(this);

		CPlayer@ local = getLocalPlayer();
		if (controls.isKeyJustPressed(controls.getActionKeyKey(AK_ACTION1)))
		{
			CBitStream params;
			params.write_u8(0);
			params.write_netid(local.getNetworkID());
			this.SendCommand(this.getCommandID("say on scores"), params);
		}
		if (controls.isKeyJustPressed(controls.getActionKeyKey(AK_ACTION2)))
		{
			CBitStream params;
			params.write_u8(1);
			params.write_netid(local.getNetworkID());
			this.SendCommand(this.getCommandID("say on scores"), params);
		}
	}
}

void onCommand(CRules@ this, u8 cmd, CBitStream @params)
{
	CPlayer@ player;
	if (getNet().isClient() && cmd == this.getCommandID("say on scores"))
	{
		const u8 type = params.read_u8();
		CPlayer@ player = getPlayerByNetworkId(params.read_netid());
		if (player is null)
			return;

		if(player.isMyPlayer())
		{
			Sound::Play("buttonclick");
		}

		Campaign::Portrait@ p = getPortraitOfPlayer(player, _portraits);
		if (p is null)
			return;
		p.chatTimer = 1.0f;
		p.chat = type == 0 ? _positiveChats[ XORRandom(_positiveChats.length) ]
		         : _negativeChats[ XORRandom(_negativeChats.length) ];
	}

}

void UpdatePortrait(Campaign::Portrait@ p, CPlayer@ player)
{
	p.name = player.getCharacterName();
	p.team = player.getTeamNum();
	p.icon = getCharacterFor(p.team, player.getClassNum()).frame;
	@p.player = player;

	Award@[]@ awards = getAwards(getRules());
	if (awards !is null && awards.length > 0)
	{
		for (uint i = awards.length - 1; i > 0; i--)
		{
			Award@ award = awards[i];
			if (award.who is player)
			{
				if (p.fields.length >= 3)
				{
					p.fields.removeAt(0);
				}
				p.fields.push_back(award.what);
			}
		}
	}
}

void UpdatePlayerPortraits(CRules@ this)
{
	for (uint i = 0; i < getPlayerCount(); i++)
	{
		CPlayer@ player = getPlayer(i);

		// update existing portrait

		bool portraitFound = false;
		for (uint j = 0; j < _portraits.length; j++)
		{
			Campaign::Portrait@ p = _portraits[j];
			if (p.player is player)
			{
				UpdatePortrait(p, player);
				//printf("update portrait " + j + " " + player.getCharacterName());
				portraitFound = true;
				break;
			}
		}

		// create new portrait if not found
		if (!portraitFound)
		{
			if (player.getTeamNum() == 255 || player.getClassNum() == 255)
				continue;

			Campaign::Portrait p;
			@p.player = player;
			//printf("create portrait " + _portraits.length + " " + player.getCharacterName() + " i " + i);
			UpdatePortrait(p, player);
			_portraits.push_back(p);
		}
	}

	// remove portraits if player left

	for (uint j = 0; j < _portraits.length; j++)
	{
		Campaign::Portrait@ p = _portraits[j];
		bool foundPlayer = false;
		for (uint i = 0; i < getPlayerCount(); i++)
		{
			CPlayer@ player = getPlayer(i);
			if (p.player is player)
			{
				foundPlayer = true;
				break;
			}
		}
		if (!foundPlayer)
		{
			//printf("remove portrait " + j + " " + p.name);
			_portraits.removeAt(j--);
		}
	}
}

Campaign::Portrait@ getPortraitOfPlayer(CPlayer@ player, Campaign::Portrait[]@ portraits)
{
	for (uint i = 0; i < portraits.length; i++)
	{
		if (portraits[i].player is player)
			return portraits[i];
	}
	return null;
}

f32 getTimerPercentage(Game::Timer@ timer, const f32 start, const f32 end = -1.0f)
{
	if (timer !is null)
	{
		const u32 time = getGameTime();
		int duration = timer.duration * getTicksASecond();
		f32 timerStartTime = f32(timer.endTime - duration);
		f32 timeStart = f32(timerStartTime + start * duration) / f32(duration);
		f32 timeEnd = f32(timerStartTime + end * duration) / f32(duration);
		f32 timeCurrent = f32(time - timerStartTime) / f32(duration);
		if (timeCurrent < timeStart)
			return 0.0f;
		if (timeCurrent > timeEnd)
			return 1.0f;
		if ((end - start) > 0.0f && end > 0.0f)
			return (timeCurrent - start) / (end - start);
		else
			return 0.0f;
	}
	return 1.0f;
}

//todo move to include
void DrawTRGuiFrame(Vec2f upper, Vec2f lower)
{
	GUI::DrawRectangle(upper - Vec2f(2, 2), lower + Vec2f(2, 2), Colours::BLACK);
	GUI::DrawRectangle(upper - Vec2f(1, 1), lower + Vec2f(1, 1), Colours::DARK);
	GUI::DrawRectangle(upper, lower, Colours::BLACK);
}

void onRender(CRules@ this)
{
	if (this.get_s16("in menu") != 0)
		return;

	Campaign::Data@ data = Campaign::getCampaign(this);

	Driver@ driver = getDriver();
	Vec2f screenDim = driver.getScreenDimensions();
	Vec2f screenCenter = driver.getScreenCenterPos();
	const u32 time = getGameTime();

	// TEAM PRESENTATION

	if (this.isIntermission())
	{
		Game::Timer@ timer = Game::getTimer("teams");

		//ClearScreen( this );
		DrawTRGuiFrame(Vec2f(screenDim.x / 3, 0), Vec2f(2 * screenDim.x / 3, screenDim.y));

		GUI::SetFont("menu");
		Vec2f teamNamePos(screenDim.x * 0.5f, screenDim.y * 0.2f);
		Vec2f scorePos = screenCenter;
		const bool seriesEnded = Campaign::isSeriesEnded(data);

		// team names or win msg

		if (seriesEnded) // win msg
		{
			const int team1score = Campaign::getTeamScore(data, 0);
			const int team2score = Campaign::getTeamScore(data, 1);
			SColor color = color_white;
			string winmsg = "It's a tie";
			if (team1score > team2score)
			{
				winmsg = "Beer Nation wins!";
				color = Colours::TEAM1;
			}
			else if (team1score < team2score)
			{
				winmsg = "Wine Republic wins!";
				color = Colours::TEAM2;
			}

			GUI::DrawTextCentered(winmsg, Vec2f(screenDim.x * 0.5f, teamNamePos.y + Maths::Sin(1.5f * time * 5) * 2.5f), getTimerPercentage(timer, 0.0f, 0.02f) < 1.0f ? color_black : color);
		}
		else // names
		{
			//teamNamePos.y += -getTimerPercentage(timer, 0.4f, 0.6f) * screenDim.y;
			GUI::DrawTextCentered("BEER", Vec2f(screenDim.x * 0.4f, teamNamePos.y), getTimerPercentage(timer, 0.0f, 0.025f) < 1.0f ? color_black : Colours::TEAM1);
			GUI::DrawTextCentered("WINE", Vec2f(screenDim.x * 0.6f, teamNamePos.y), getTimerPercentage(timer, 0.0f, 0.05f) < 1.0f ? color_black : Colours::TEAM2);
		}

		// score

		if (getTimerPercentage(timer, 0.0f, 0.1f) >= 1.0f)
		{
			const int team1score = Campaign::getTeamScore(data, 0);
			const int team2score = Campaign::getTeamScore(data, 1);
			//scorePos.y += getTimerPercentage(timer, 0.2f, 0.5f) * screenDim.y;
			GUI::DrawTextCentered(":", scorePos, color_white);
			GUI::DrawTextCentered("" + Maths::Floor(getTimerPercentage(timer, 0.1f, 0.15f) * f32(team1score)), Vec2f(scorePos.x - 50, scorePos.y), Colours::TEAM1);
			GUI::DrawTextCentered("" + Maths::Floor(getTimerPercentage(timer, 0.15f, 0.2f) * f32(team2score)), Vec2f(scorePos.x + 50, scorePos.y), Colours::TEAM2);
		}

		// maps


		if (getTimerPercentage(timer, 0.0f, 0.2f) >= 1.0f)
		{
			const uint battlesCount = data.battles.length;
			const int middleOffset = 0;
			Vec2f dimensions(battlesCount * ICON_DISPLAY_SIZE.x, ICON_DISPLAY_SIZE.y);
			Vec2f ul = Vec2f(0.0f, screenCenter.y * 0.8f) + screenCenter - dimensions / 2.0f + Vec2f(middleOffset * ICON_DISPLAY_SIZE.x, 0.0f);
			Vec2f lr = ul + dimensions;
			Vec2f battlePos(ul.x + ICON_DISPLAY_SIZE.x / 2.0f, (ul.y + lr.y) / 2.0f);

			DrawTRGuiFrame(ul - Vec2f(8, 8), lr + Vec2f(8, 8));

			for (uint locIt = 0; locIt < battlesCount; locIt++)
			{
				const string token = getLocationToken(data.battles[locIt]);
				SColor color = data.team[locIt] == -1 ? SColor(0) : (data.team[locIt] == 0 ? Colours::TEAM1 : Colours::TEAM2);
				if (data.battleIndex == locIt && time % 40 < 21)
				{
					color = Colours::WHITE;
					if (seriesEnded)
					{
						color = color_black;
					}
				}

				GUI::DrawRectangle(battlePos - ICON_SIZE / 1.6f, battlePos + ICON_SIZE / 1.6f, color);
				GUI::DrawIconByName(token, battlePos - ICON_SIZE / 2.0f, SCALE / 2.0f);

				// mark as done
				if (locIt < data.battleIndex || seriesEnded)
				{
					GUI::DrawIconByName("$location_done$", battlePos - ICON_SIZE / 2.0f, SCALE / 2.0f);
				}
				battlePos.x += ICON_DISPLAY_SIZE.x;
			}
		}

		//if (getTimerPercentage(timer, 0.0f, 0.2f) >= 1.0f)
		{
			const s32 team_width = 160;
			const s32 team_margin = 4;

			GUI::SetFont("gui");
			const f32 portraitVerticalSpace = 62;
			Vec2f team1pos(screenDim.x * 0.05f, 0.0f + portraitVerticalSpace / 2);
			Vec2f team2pos(screenDim.x * 0.95f, team1pos.y);
			for (uint i = 0; i < _portraits.length; i++)
			{
				Campaign::Portrait@ p = _portraits[i];

				const f32 p_bounce_amount =  p.chatTimer > 0.0f ? (Maths::Sin(1.0f * (time + i * 13)) * 5) * 0.5f : 0.0f;

				if (i > 0)
				{
					Campaign::Portrait@ prev = _portraits[i - 1];
					if (prev.showTimer < 1.0f)
						continue;
				}

				// manage show timer

				p.showTimer += 0.5f;

				// set vertical pos

				Vec2f iconpos;
				if (p.team == 0)
				{
					iconpos = team1pos;
					team1pos.y += portraitVerticalSpace;
				}
				else
				{
					iconpos = team2pos;
					team2pos.y += portraitVerticalSpace;
				}

				// radio

				{
					Vec2f upper = (iconpos - _portraitSize * 0.5f) + Vec2f(-team_margin, 8);
					Vec2f lower =  upper + Vec2f(team_width, _portraitSize.y);
					if (p.team == 1)
					{
						upper.x -= team_width - _portraitSize.x - team_margin * 2;
						lower.x -= team_width - _portraitSize.x - team_margin * 2;
					}

					DrawTRGuiFrame(upper, lower);
				}

				GUI::DrawIcon(_portraits_file, p.icon, _portraitSize, iconpos - _portraitSize * 0.5f + Vec2f(0.0f, p_bounce_amount), 0.5f);

				// name

				if (p.team == 0)
				{
					Vec2f textpos = iconpos + Vec2f(_portraitSize.x * 0.5f, -_portraitSize.y * 0.5f + 4);
					GUI::DrawText(p.name, textpos, Colours::TEAM1);
				}
				else
				{
					Vec2f nameDim;
					GUI::GetTextDimensions(p.name, nameDim);
					Vec2f textpos = iconpos + Vec2f(-_portraitSize.x * 0.5f - nameDim.x - 3, -_portraitSize.y * 0.5f + 4);
					GUI::DrawText(p.name, textpos, Colours::TEAM2);
				}
				//printf("p.name " + i + " " + p.name);

				// awards

				if (p.chatTimer <= 0.0f)
				{
					//printf("p.showTimer " + p.showTimer);
					for (uint fieldsCounter = 0; fieldsCounter < p.fields.length; fieldsCounter++)
					{
						if (p.showTimer < 8.0f + f32(fieldsCounter) / 3.0f)
							continue;
						f32 fieldsX = portraitVerticalSpace;
						f32 fieldsY = iconpos.y - _portraitSize.y * 0.5f + fieldsCounter * 8;
						if (p.team == 0)
						{
							GUI::DrawText(p.fields[fieldsCounter], Vec2f(fieldsX, fieldsY), color_white);
						}
						else
						{
							Vec2f nameDim;
							GUI::GetTextDimensions(p.fields[fieldsCounter], nameDim);
							GUI::DrawText(p.fields[fieldsCounter], Vec2f(screenDim.x - nameDim.x - fieldsX - 4, fieldsY), color_white);
						}
					}
				}

				// chat

				if (p.chatTimer > 0.0f)
				{
					string chat = "'" + p.chat + "'";
					Vec2f nameDim;
					GUI::GetTextDimensions(chat, nameDim);
					const f32 xd = (1.0f - p.chatTimer * 0.5f);
					if (p.team == 0)
					{
						GUI::DrawTextCentered(chat, Vec2f(xd * screenDim.x / 4 - nameDim.x / 2, iconpos.y), color_white);
					}
					else
					{
						GUI::DrawTextCentered(chat, Vec2f(screenDim.x - xd * screenDim.x / 4 + nameDim.x / 2, iconpos.y), color_white);
					}

					p.chatTimer -= 0.05f;
				}
			}
		}

	}

	// BRIEFING

	if (this.isWarmup())
	{
		const uint battlesCount = data.battles.length;
		const int middleOffset = data.battles.length > (screenDim.x / ICON_SIZE.x) ? Maths::Floor(data.battles.length / 2) - data.battleIndex : 0;

		Vec2f dimensions(battlesCount * ICON_DISPLAY_SIZE.x, ICON_DISPLAY_SIZE.y);
		Vec2f ul = Vec2f(0.0f, screenCenter.y * 0.8f) + screenCenter - dimensions / 2.0f + Vec2f(middleOffset * ICON_DISPLAY_SIZE.x, 0.0f);
		Vec2f lr = ul + dimensions;

		Vec2f iconPos(ul.x + ICON_DISPLAY_SIZE.x / 2.0f, (ul.y + lr.y) / 2.0f);

		// letterbox
		DrawLetterbox(this);

		//printf("locationsCount " + locationsCount);
		/*	for (uint locIt = 0; locIt < battlesCount; locIt++)
			{
				const string token = getLocationToken(data.battles[locIt]);
				SColor color = data.team[locIt] == -1 ? SColor(0) : (data.team[locIt] == 0 ? Colours::TEAM1 : Colours::TEAM2);
				if (this.isWarmup() && data.battleIndex == locIt && time % 40 < 21)
				{
					color = Colours::WHITE;
				}

				GUI::DrawRectangle(iconPos - ICON_SIZE / 1.6f, iconPos + ICON_SIZE / 1.6f, color);
				GUI::DrawIconByName(token, iconPos - ICON_SIZE / 2.0f, SCALE / 2.0f);
				iconPos.x += ICON_DISPLAY_SIZE.x;
			} */

		// ready text

		GUI::SetFont("menu");
		GUI::DrawTextCentered("RUN TO THE EDGE!",
		                      Vec2f(getScreenWidth() / 2, 40),
		                      color_white);

		Vec2f[] spawns;
		CPlayer@ player = getLocalPlayer();
		if (player !is null)
		{
			if (getMap().getMarkers(getTeamMarkerString(player.getTeamNum()), spawns))
			{
				GUI::SetFont("gui");
				GUI::DrawTextCentered("START HERE",
				                      Vec2f(getDriver().getScreenPosFromWorldPos(spawns[0]).x, getMap().tilemapheight * getMap().tilesize * 0.4f + Maths::Sin(getGameTime() * 0.25f) * 2.0f) ,
				                      player.getTeamNum() == 0 ? Colours::TEAM1 : Colours::TEAM2);
			}
		}
	}

	// GAME

	if (this.isMatchRunning())
	{
		Game::Timer@ timer = Game::getTimer("timeout");
		if (timer !is null && this.get_s16("in menu") > 0 &&
		        timer.duration - Game::getTimerSecondsLeft(timer) < 3)
		{
			GUI::SetFont("menu");
			GUI::DrawTextCentered("RUN!",
			                      Vec2f(getScreenWidth() / 2, 40),
			                      color_white);
		}
	}

	if (this.isGameOver())
	{
		DrawLetterbox(this);


		GUI::SetFont("menu");
		SColor color;
		string text = this.get_string("win msg");
		if (this.getTeamWon() == 0)
		{
			color = Colours::TEAM1;
		}
		else if (this.getTeamWon() == 1)
		{
			color = Colours::TEAM2;
		}
		else
		{
			color = Colours::WHITE;
		}

		GUI::DrawTextCentered(text,
		                      Vec2f(getScreenWidth() / 2, 40 + Maths::Sin(getGameTime() * 0.25f) * 8.0f),
		                      color);
	}
}

void DrawLetterbox(CRules@ this)
{
	Driver@ driver = getDriver();
	Vec2f screenDim = driver.getScreenDimensions();

	// letterbox
	DrawTRGuiFrame(Vec2f(0, 0), Vec2f(screenDim.x, 70));
	DrawTRGuiFrame(Vec2f(0, screenDim.y - 70), screenDim);
}

void ClearScreen(CRules@ this, SColor color = color_black)
{
	Driver@ driver = getDriver();
	Vec2f screenDim = driver.getScreenDimensions();

	GUI::DrawRectangle(Vec2f(0, 0), screenDim, color);
}


string getLocationToken(const string &in name)
{
	return "$location_" + name + "$";
}
