#include "GameColours.as"

void onRenderScoreboard(CRules@ this)
{

	//sort players
	CPlayer@[] sortedplayers;
	CPlayer@[] spectators;
	for (u32 i = 0; i < getPlayersCount(); i++)
	{
		CPlayer@ p = getPlayer(i);
		f32 kdr = p.getKills();
		bool inserted = false;
		if (p.getTeamNum() == this.getSpectatorTeamNum())
		{
			spectators.push_back(p);
			continue;
		}
		for (u32 j = 0; j < sortedplayers.length; j++)
		{
			if (sortedplayers[j].getKills() < kdr)
			{
				sortedplayers.insert(j, p);
				inserted = true;
				break;
			}
		}
		if (!inserted)
			sortedplayers.push_back(p);
	}

	//draw board

	f32 stepheight = 16;
	f32 hmargin = 20;
	Vec2f topleft(hmargin, 40);
	Vec2f bottomright(getScreenWidth() - hmargin, topleft.y + (sortedplayers.length + (spectators.length == 0 ? 0 : 2) + 3.5) * stepheight);
	GUI::DrawPane(topleft, bottomright, SColor(0xffc0c0c0));

	//offset border

	topleft.x += stepheight;
	bottomright.x -= stepheight;
	topleft.y += stepheight;

	//draw spectators

	GUI::SetFont("gui");

	if (spectators.length > 0)
	{
		f32 specy = bottomright.y - stepheight * 2;
		GUI::DrawLine2D(Vec2f(topleft.x, specy), Vec2f(bottomright.x, specy), SColor(0xff404040));

		Vec2f textdim;
		string s = "Spectators:";
		GUI::GetTextDimensions(s, textdim);

		GUI::DrawText(s, Vec2f(topleft.x, specy), SColor(0xffaaaaaa));

		f32 specx = topleft.x + textdim.x + 10;
		for (u32 i = 0; i < spectators.length; i++)
		{
			CPlayer@ p = spectators[i];
			if (specx < bottomright.x - 100)
			{
				string name = p.getCharacterName();
				if (i != spectators.length - 1)
					name += ",";
				GUI::GetTextDimensions(name, textdim);
				GUI::DrawText(name, Vec2f(specx, specy), getTeamColor(p.getTeamNum()));
				specx += textdim.x + 10;
			}
			else
			{
				GUI::DrawText("and more ...", Vec2f(specx, specy), SColor(0xffaaaaaa));
				break;
			}
		}
	}

	//draw player table header

	const f32 boxwidth = (bottomright.x-topleft.x);
	GUI::DrawText("Player", Vec2f(topleft.x, topleft.y), SColor(0xffffffff));
	GUI::DrawText("Ping", Vec2f(topleft.x + boxwidth*0.5f, topleft.y), SColor(0xffffffff));
	GUI::DrawText("Kills", Vec2f(topleft.x + boxwidth*0.7f, topleft.y), SColor(0xffffffff));
	GUI::DrawText("Deaths", Vec2f(topleft.x + boxwidth*0.9f, topleft.y), SColor(0xffffffff));
//	GUI::DrawText("KDR", Vec2f(bottomright.x - 100, topleft.y), SColor(0xffffffff));

	topleft.y += stepheight * 0.5f;

	CControls@ controls = getControls();
	Vec2f mousePos = controls.getMouseScreenPos();

	//draw players
	for (u32 i = 0; i < sortedplayers.length; i++)
	{
		CPlayer@ p = sortedplayers[i];

		topleft.y += stepheight;
		bottomright.y = topleft.y + stepheight;

		Vec2f lineoffset = Vec2f(0, 0);

		u32[] teamcolours = {0xff6666ff, 0xffff6666};
		u32 playercolour = (p.getBlob() is null || p.getBlob().hasTag("dead")) ? 0xffaaaaaa :
		                   teamcolours[p.getBlob().getTeamNum() % teamcolours.length];

		GUI::DrawLine2D(Vec2f(topleft.x, bottomright.y + 1) + lineoffset, Vec2f(bottomright.x, bottomright.y + 1) + lineoffset, SColor(0xff404040));
		GUI::DrawLine2D(Vec2f(topleft.x, bottomright.y) + lineoffset, bottomright + lineoffset, SColor(playercolour));

		string tex = "";
		u16 frame = 0;
		Vec2f framesize;
		if (p.isMyPlayer())
		{
			tex = "ScoreboardIcons.png";
			frame = 4;
			framesize.Set(16, 16);
		}
		else
		{
			// tex = p.getScoreboardTexture();
			// frame = p.getScoreboardFrame();
			// framesize = p.getScoreboardFrameSize();
			tex = "HoverIcons.png";
			frame = p.getClassNum();
			framesize.Set(16, 16);
		}
		if (tex != "")
		{
			GUI::DrawIcon(tex, frame, framesize, topleft, 0.5f, p.getTeamNum());
		}

		string playername = "";
		if (mousePos.y > topleft.y && mousePos.y < topleft.y + 15)
		{
			playername = " " + p.getUsername();
		}
		else
		{
			playername = p.getCharacterName();
			string clantag = p.getClantag();
			if (clantag.length > 0)
			{
				playername = clantag + " " + playername;
			}
		}

		//have to calc this from ticks
		s32 ping_in_ms = s32(p.getPing() * 1000.0f / 30.0f);


		//render the player + stats
		GUI::DrawText(playername, topleft + Vec2f(20, 0), getTeamColor(p.getTeamNum()));

		GUI::DrawText("" + ping_in_ms, Vec2f(topleft.x + boxwidth*0.5f, topleft.y), SColor(0xffffffff));
		GUI::DrawText("" + p.getKills(), Vec2f(topleft.x + boxwidth*0.7f, topleft.y), SColor(0xffffffff));
		GUI::DrawText("" + p.getDeaths(), Vec2f(topleft.x + boxwidth*0.9f, topleft.y), SColor(0xffffffff));
		//GUI::DrawText("" + formatFloat(getKDR(p), "", 3, 1), Vec2f(bottomright.x - 100, topleft.y), SColor(0xffffffff));
	}
}
