#include "GameColours.as"
#include "UI.as"
#include "TRChatCommon.as"
#include "SoldierCommon.as"
#include "PlayerStatsCommon.as"

#define CLIENT_ONLY

//set up the chat console properly
void onInit(CRules@ this)
{
	SetChatVisible(false);
	PositionChat(true);

	cl_chatbubbles = false;
}

void PositionChat(bool top)
{
	chat_on_top = top;

	Driver@ driver = getDriver();
	if (driver is null) return;

	f32 width = driver.getScreenWidth();
	f32 height = driver.getScreenHeight();

	f32 chat_width = 350;
	f32 chat_halfwidth = chat_width * 0.5f;
	f32 chat_height = 200;

	Vec2f bottom_center(width * 0.5f, height * 0.5f + chat_height * 0.5f);

	SetChatLayout(bottom_center + Vec2f(-chat_halfwidth, -chat_height), bottom_center + Vec2f(chat_halfwidth, 0));
}

//helper - chops into lines
string[] chopChatLines(string &in text_in)
{
	string line = text_in;
	string[] lines;

	s32 line_cutoff = 30;
	while (line.size() > 0)
	{
		if (line.size() < line_cutoff)
		{
			lines.push_back(line);
			line = "";
		}
		else
		{
			//zoom to last space + handle no spaces nicely enough
			uint position = line_cutoff;
			bool foundspace = false;
			for (uint i = 0; i < line_cutoff; i++)
			{
				if (line[i] == 0x20) //space
				{
					position = i;
					foundspace = true;
				}
			}

			lines.push_back(line.substr(0, position));
			line = line.substr(position + (foundspace ? 1 : 0));
		}
	}

	return lines;
}

bool radioflip_oscillate = false;
bool isRadioFlipped(CRules@ this, CPlayer@ player)
{
	//team for campaign
	if (sv_gamemode == "Campaign")
	{
		return (player.getTeamNum() == 1);
	}
	//otherwise oscillate
	radioflip_oscillate = !radioflip_oscillate;
	return radioflip_oscillate;
}

//radio chatter vars
class RadioChatter
{
	u32 timer;
	string username;
	string chat1;
	string chat2;
	u32 icon;
	bool flipside;
	u8 team;
};

RadioChatter[] chat;
bool chat_on_top = true;
const f32 chat_swap_thresh = 0.4f;
const u32 min_chat_duration = getTicksASecond() * 2.3;
const u32 max_chat_duration = getTicksASecond() * 5;

string getFullName(CPlayer@ player)
{
	string clantag = player.getClantag();
	return (clantag.size() > 0 ? clantag + " " : "") + player.getCharacterName();
}

bool onClientProcessChat(CRules@ this, const string& in text_in, string& out text_out, CPlayer@ player)
{
	text_out = text_in;

	string[] lines = chopChatLines(text_in);

	string username = "MSG";
	u8 icon = 0;
	s32 team = 255;
	bool flipped = false;

	if (player !is null)
	{
		username = getFullName(player);
		flipped = isRadioFlipped(this, player);

		icon = lookupIcon(player.getUsername(), getTRChatNameIconLookup());
		team = player.getTeamNum();
	}

	while (lines.length > 0)
	{
		RadioChatter rc;

		rc.username = username;
		//first line guaranteed
		rc.chat1 = lines[0];
		lines.removeAt(0);

		rc.chat2 = "";
		if (lines.length > 0)
		{
			rc.chat2 = lines[0];
			lines.removeAt(0);
		}

		string thischat = rc.chat1 + rc.chat2;
		rc.timer = Maths::Min(max_chat_duration, min_chat_duration + thischat.size() * 3.5f);

		rc.icon = icon;
		rc.flipside = flipped;
		rc.team = team;

		chat.push_back(rc);
	}

	// stats
	Stats@ stats = getStats(player);
	if (stats !is null)
	{
		stats.chatCharactersCount += text_in.size();
	}


	return true;
}

//keep the chat hidden the rest of the time

void onEnterChat(CRules@ this)
{
	SetChatVisible(true);
	PositionChat(chat_on_top);
}

void onExitChat(CRules@ this)
{
	SetChatVisible(false);
}

//render and slowly pop out the radio chatter

//TODO: sound for the radio

void onTick(CRules@ this)
{
	//dont update if we have menus; means we dont miss anything
	if (UI::hasAnyContent())
		return;

	if (chat.length > 0)
	{
		chat[0].timer--;
		if (chat[0].timer <= 0)
		{
			chat.removeAt(0);
		}
	}

	//dynamic positioning
	{
		CBlob@ b = getLocalPlayerBlob();
		if(this.isWarmup())
		{
			chat_on_top = false;
		}
		else
		{
			CMap@ map = getMap();
			if (b !is null && map !is null)
			{
				f32 b_height = b.getPosition().y / (map.tilemapheight * map.tilesize);
				if (chat_on_top)
				{
					if (b_height < chat_swap_thresh)
					{
						PositionChat(false);
					}
				}
				else
				{
					if (b_height > 1.0f - chat_swap_thresh)
					{
						PositionChat(true);
					}
				}
			}
		}
	}
}

void onRender(CRules@ this)
{
	if (UI::hasAnyContent())
		return;

	Driver@ driver = getDriver();
	if (driver is null) return;

	if (chat.length == 0) return;

	RadioChatter@ rc = chat[0];

	f32 width = driver.getScreenWidth();
	f32 height = driver.getScreenHeight();

	Vec2f top_center(width * 0.5f, 0.0f);
	f32 radio_width = 300.0f;
	f32 radio_halfwidth = radio_width * 0.5f;
	f32 radio_height = 50.0f;

	//offset horizontally
	f32 offset_amount = radio_width * 0.1f;
	if (!rc.flipside)
	{
		top_center.x -= offset_amount;
	}
	else
	{
		top_center.x += offset_amount;
	}

	f32 yoffset;
	//flip top or bottom
	if (!chat_on_top)
	{
		top_center.y += (height - radio_height);
		yoffset = 4;
	}
	else
	{
		yoffset = 4 + (this.get_string("scrolling text") != "" ? 20 : 0);
	}

	Vec2f top_left = top_center + Vec2f(-radio_halfwidth, yoffset - 4);
	Vec2f bottom_right = top_center + Vec2f(radio_halfwidth, radio_height + yoffset - 4);

	GUI::DrawRectangle(top_left + Vec2f(-1, -1),
	                   bottom_right + Vec2f(1, 1),
	                   Colours::BLACK);
	GUI::DrawRectangle(top_left,
	                   bottom_right,
	                   Colours::DARK);

	Vec2f chat_top = top_left + Vec2f(64, 0);

	Vec2f icon_size = Vec2f(48, 48);
	Vec2f icon_offset = Vec2f(8, 4);
	Vec2f icon_top = top_left + icon_offset;
	if (rc.flipside)
	{
		chat_top = top_left + Vec2f(8, 0);
		icon_top.x = bottom_right.x - icon_size.x - icon_offset.x;
	}

	if (!chat_on_top)
	{
		icon_top.y = bottom_right.y - icon_size.y - icon_offset.y;
	}

	GUI::SetFont("hud");

	GUI::DrawText(rc.username, chat_top, getTeamColor(rc.team));
	GUI::DrawText(rc.chat1, chat_top + Vec2f(0, 12), Colours::WHITE);
	GUI::DrawText(rc.chat2, chat_top + Vec2f(0, 24), Colours::WHITE);

	GUI::DrawRectangle(icon_top + Vec2f(-1, -1),
	                   icon_top + Vec2f(49, 49),
	                   Colours::BLACK);
	GUI::DrawIcon("Sprites/UI/radio_portraits.png", rc.icon, Vec2f(48, 48), icon_top, 0.5f);

}

