#define CLIENT_ONLY

#include "DrawScores.as"
#include "Menus.as"
#include "Timers.as"
#include "GamemodeCommon.as"
#include "SoldierCommon.as"
#include "GameColours.as"
#include "UI.as"

const string _hud_file = "Sprites/UI/hud_parts.png";

void onRender(CRules@ this)
{
	if (this.hasTag("hide actor hud") || this.get_s16("in menu") > 0 || UI::hasAnyContent())
		return;

	CMap@ map = getMap();
	CPlayer@ player = getLocalPlayer();
	if (map is null || player is null)
		return;

	const u8 state = this.getCurrentState();

	/////////////////////////////////
	//actual rendering code
	/////////////////////////////////

	if (!isFreeBuild())
	{
		//lower hud
		Vec2f lower_hud_frame = Vec2f(170, 17);
		Vec2f lower_hud_ul(getScreenWidth() / 2 - lower_hud_frame.x * 0.5f, getScreenHeight() - lower_hud_frame.y);
		{
			//draw background
			GUI::DrawIcon(_hud_file, 0, lower_hud_frame, lower_hud_ul, 0.5f);
		}

		{
			//draw coins
			GUI::SetFont("hud");
			GUI::DrawText("COINS: " + player.getCoins(),
			              lower_hud_ul + Vec2f(8, 0),
			              color_white);

		}

		Vec2f textPos = getDriver().getScreenCenterPos() + Vec2f(0, -50 + 2 * Maths::Sin(getGameTime() * 0.2f));

		// DEBUG STUFF
		if (getNet().isServer())
		{
			GUI::SetFont("irrlicht");
			GUI::DrawText("players: " + this.get_u32("players in building") + "/" + this.get_u32("players_required"),
			              lower_hud_ul + Vec2f(116, 16),
			              color_white);
			GUI::DrawText("state: " + this.getCurrentState(),
			              lower_hud_ul + Vec2f(116, 0),
			              color_white);
		}
	}

	// player indicator

	CCamera@ camera = getCamera();
	const u32 time = getGameTime();

	CBlob@ blob = player.getBlob();
	if (blob is null)
	{
		return;
	}
	Soldier::Data@ data = Soldier::getData(blob);

	const bool myplayer = blob.isMyPlayer();
	const int controlIndex = blob.getMyPlayerIndex();
	if (!blob.isChatBubbleVisible())
	{
		Vec2f player_pos = getDriver().getScreenPosFromWorldPos(data.pos
		                   + Vec2f(0, -28 + (myplayer ? Maths::Sin(0.5f * time) * 2.5f : 0.0f)));
		const u8 our_class = 5;
		GUI::DrawIcon("Sprites/HoverIcons.png", our_class, Vec2f(16, 16),
		              player_pos - Vec2f(16, 16)*camera.targetDistance, camera.targetDistance,
		              color_white);
	}

	// player names

	GUI::SetFont("intro");
	CBlob@[] all;
	getBlobsByTag("player", @all);
	for (u32 i = 0 ; i < all.length; i++)
	{
		CBlob@ b = all[i];
		if (b.getPlayer() !is null)
		{
			Vec2f pos = Vec2f(b.getScreenPos().x, 227 + 12 + (i % 6) * 11);
			GUI::DrawTextCentered(b.getPlayer().getCharacterName(), pos, Colours::WHITE);
		}
	}
}
