#include "Timers.as"

APIServer@[] _servers;
string _gridIcons = "Sprites/UI/gridicons.png";
Vec2f _gridIconSize(64, 128);
const bool TEST_SERVERS = false;

string _joinAddress;

void ShowGridBrowserInit(CRules@ this, UI::Group@ group, UI::Control@ control)
{
	UI::Clear();
	if (!TEST_SERVERS && !Engine::isAuthenticated())
	{
		Engine::ShowLoginWindow();
		getHUD().ShowCursor();
		return;
	}

	if (group !is null)
	{
		UI::Transition(group, Vec2f(-1.0f, 0.0f));
		UI::Transition(control, Vec2f(1.0f, 0.0f));
		UI::Clear(group.name);
	}
	UI::Clear("title");
	UI::Clear("multi");
	UI::Clear("multigrid");

	if (!TEST_SERVERS)
	{
		CScriptedBrowser@ browser = getBrowser();
		browser.ping = true;
		browser.filter = "";
		browser.RequestList();

		UI::AddGroup("servers downloading", Vec2f(0.0f, 0.0), Vec2f(1, 1));
		UI::Grid(1, 1);
		UI::Label::Add("Downloading servers list...");
	}
	else
	{
		ShowGridBrowser(this, _servers);
	}
}

void OnRequestList(CRules@ this)
{
	CScriptedBrowser@ browser = getBrowser();
	_servers.clear();
	browser.getServersList(_servers);

	ShowGridBrowser(this, _servers);
}

void ShowGridBrowser(CRules@ this, APIServer@[]@ servers)
{
	UI::Clear("servers downloading");

	AddGridBrowser(servers);

	UI::AddGroup("multi", Vec2f(0.2f, 0.8f), Vec2f(0.8f, 0.99f));
	UI::SetFont("hud");
	UI::Grid(2, 1);
	UI::SetLastSelection();
	UI::SetSmallSelector();
	UI::Button::Add("Back", _backCallback);
	UI::Label::Add("to main menu");
	UI::Transition(Vec2f(0.0f, -1.0f));
}

// grid button

const string _serverIcons = "Sprites/UI/lobby_icons.png";
Vec2f _serverIconSize(48, 48);

void GridButtonRender(UI::Proxy@ proxy)
{
	if (proxy !is null && proxy.control !is null && proxy.frames.length > 0)
	{
		GUI::DrawIcon(proxy.image, proxy.frames[0], proxy.imageSize, (proxy.ul + proxy.lr) / 2 - proxy.imageSize / 2, 0.5f);

		int serverIcon;
		proxy.control.vars.get("icon", serverIcon);

		GUI::DrawIcon(_serverIcons, serverIcon, _serverIconSize, (proxy.ul + proxy.lr) / 2 - _serverIconSize / 2 + Vec2f(0.0f, -8.0f), 0.5f);
	}
}

void GridRenderCaption(UI::Proxy@ proxy)
{
	if (proxy.control !is null)
	{
		Vec2f dim, pos;
		string players;
		string caption2;

		// playres count

		if (!GUI::isFontLoaded("grid_pcount"))
		{
			GUI::LoadFont("grid_pcount", "GUI/Fonts/grid/pcount.ttf", 8, true);
		}
		if (!GUI::isFontLoaded("grid_pcount_big"))
		{
			GUI::LoadFont("grid_pcount_big", "GUI/Fonts/grid/pcount_big.ttf", 12, true);
		}

		proxy.control.vars.get("players", players);
		string caption = "" + players;
		GUI::GetTextDimensions(caption, dim);
		pos.x = (proxy.ul.x + proxy.lr.x) / 2 - 2;
		pos.y = proxy.ul.y - 5;

		if(proxy.selected)
		{
			GUI::SetFont("grid_pcount_big");
			pos.y -= 2;
		}
		else
		{
			GUI::SetFont("grid_pcount");
		}

		GUI::DrawTextCentered(caption, pos, proxy.disabled ? UI::CAPTION_DISABLE_COLOUR : proxy.selected ? UI::CAPTION_HOVER_COLOR : UI::CAPTION_COLOR);

		// server name
		GUI::SetFont("grid_pcount_big");

		proxy.control.vars.get("caption1", caption);
		proxy.control.vars.get("caption2", caption2);

		GUI::GetTextDimensions(proxy.caption, dim);
		pos = proxy.lr;
		pos.x = (proxy.ul.x + proxy.lr.x) / 2 - 2;
		pos.y -= dim.y/2;
		GUI::DrawTextCentered(caption, pos, proxy.disabled ? UI::CAPTION_DISABLE_COLOUR : proxy.selected ? UI::CAPTION_HOVER_COLOR : UI::CAPTION_COLOR);

		pos.y += dim.y - 4;
		GUI::DrawTextCentered(caption2, pos, proxy.disabled ? UI::CAPTION_DISABLE_COLOUR : proxy.selected ? UI::CAPTION_HOVER_COLOR : UI::CAPTION_COLOR);

		// description
		if (proxy.selected)
		{
			string description;
			proxy.control.vars.get("description", description);
			pos.x = getDriver().getScreenWidth() / 2;
			pos.y = 5.0f;
			GUI::DrawTextCentered(description, pos, UI::CAPTION_HOVER_COLOR);
		}
	}
}

void AddGridBrowser(APIServer@[]@ servers)
{
	UI::AddGroup("multigrid", Vec2f(0.05f, 0.4f), Vec2f(0.95f, 0.6f), _gridIcons, _gridIconSize);
	UI::SetFont("intro");
	UI::Grid(_testservers.length, 1);
	UI::SetGridSelector();

	// count servers


	if (TEST_SERVERS)
	{
		AddTestServers();
	}
	else
	{
		TestServer@[] found_servers;

		for (uint i = 0; i < servers.length; i++)
		{
			APIServer@ server = servers[i];
			string address_and_port = server.serverIPv4Address + ":" + server.serverPort;
			printf(i + " " + address_and_port + " " + server.gameMode + " " + server.serverName + " | " + server.description);

			// filter from testservers
			TestServer @testServer;
			for (uint i = 0; i < _testservers.length; i++)
			{
				TestServer @test = _testservers[i];
				if (test.address_and_port == address_and_port)
				{
					@testServer = test;
					break;
				}
			}

			if (testServer is null)
				continue;

			// players count
			string playersStr = server.description.substr(server.description.find("PLAYERS: ") + 9, server.description.length);

			testServer.players = playersStr;

			found_servers.push_back(testServer);
		}

		while (!found_servers.empty())
		{
			// select in order of testservers
			TestServer @testServer = null;
			for (uint i = 0; i < _testservers.length && testServer is null; i++)
			{
				TestServer @aim = _testservers[i];
				for (uint j = 0; j < found_servers.length && testServer is null; j++)
				{
					TestServer @test = found_servers[j];
					if (test.address_and_port == aim.address_and_port)
					{
						@testServer = test;
						found_servers.removeAt(j);
					}
				}
			}

			if (testServer is null)
				break;

			UI::Control@ serverButton = UI::Button::Add("", testServer.selectFunc, GridButtonRender, GridRenderCaption, false, 0.5f, testServer.background);

			serverButton.vars.set("address", testServer.address_and_port);

			serverButton.vars.set("caption1", testServer.caption1);
			serverButton.vars.set("caption2", testServer.caption2);

			serverButton.vars.set("icon", testServer.icon);
			serverButton.vars.set("description", testServer.description);

			serverButton.vars.set("players", testServer.players);
		}
	}

	UI::Transition(Vec2f(0.0f, 1.0f));
}

void SelectServer(CRules@ this, UI::Group@ group, UI::Control@ control)
{
	print("Browser join: " + control.caption);
	control.vars.get("address", _joinAddress);
	@group.data.escape_func = ExitAndConnect;
}

void ShowConnectTo(CRules@ this, UI::Group@ group, UI::Control@ control)
{
	@_backCallback = ShowGridBrowserInit;
	Vec2f select_area_tl(0.2f, 0.4f);
	Vec2f select_area_br(0.8f, 0.9f);
	UI::Transition(group, Vec2f(-1.0f, 0.0f));
	UI::Transition(control, Vec2f(1.0f, 0.0f));
	UI::Clear();
	UI::AddGroup("connect", select_area_tl, select_area_br);
	UI::SetFont("hud");
	UI::Grid(2, 4);
	UI::TextInput::Add(cl_joinaddress, SetJoinIPPort, false, 32);
	UI::Label::Add("IP:Port");
	UI::TextInput::Add(cl_password, SetJoinPassword, true, 32);
	UI::Label::Add("optional password");
	UI::Button::Add("Connect", SelectConnect);
	UI::Label::Add("join the server");
	UI::Button::Add("Back", _backCallback);
	UI::Transition(Vec2f(-1.0f, 0.0f));
	UI::SetLastSelection();
}

bool ExitAndConnect(CRules@ this)
{
	CNet@ net = getNet();
	cl_joinaddress = _joinAddress;
	return net.SafeConnect(_joinAddress);
}


class TestServer
{
	string address_and_port;
	string caption1, caption2;
	u32 background, icon;
	string description;
	string players;
	SELECT_FUNCTION@ selectFunc;
	TestServer(string _address_and_port,
	           string _caption1, string _caption2,
	           u32 _background,
	           u32 _icon_frame,
	           string _description,
	           SELECT_FUNCTION@ _selectFunc)
	{
		address_and_port = _address_and_port;
		caption1 = _caption1;
		caption2 = _caption2;
		background = _background;
		icon = _icon_frame;
		description = _description;
		@selectFunc = _selectFunc;
		players = "";
	}
};

TestServer[] _testservers =
{
	TestServer("178.63.17.152:50350", "NOVICE", "", 1, 0,
	"Learn the basics of Trench Run here!  No entry cost!", SelectServer),

	TestServer("178.63.17.152:50360", "REGULAR", "", 1, 1,
	"Play low-stakes Trench Run here!  Small entry cost!", SelectServer),

	TestServer("178.63.17.152:50560", "MASTER", "", 1, 2,
	"Pro-level Trench Run!  Large entry cost!", SelectServer),

	TestServer("178.63.17.152:44261", "SHOP", "", 2, 6,
	"Buy your shit here.", SelectServer),

	TestServer("", "DIRECT", "", 3, 3,
	"Connect to a server IP:port address", ShowConnectTo)
};

void AddTestServers()
{
	for (uint i = 0; i < _testservers.length; i++)
	{
		TestServer@ t = _testservers[i];

		//TODO: icon file and icon frame used here	; same with description (popup/tooltip)
		UI::Control@ serverButton = UI::Button::Add("", t.selectFunc, GridButtonRender, GridRenderCaption, false, 0.5f, t.background);

		serverButton.vars.set("address", t.address_and_port);
		serverButton.vars.set("players", 0);

		serverButton.vars.set("caption1", t.caption1);
		serverButton.vars.set("caption2", t.caption2);

		serverButton.vars.set("icon", t.icon);
		serverButton.vars.set("description", t.description);
	}
}

int getServerIcon(APIServer@ server)
{
	if (server.serverName.find("TR Official") > -1)
	{
		if (server.serverName.find("NOVICE") > -1)
		{
			return 1;
		}
		else if (server.serverName.find("REGULAR") > -1)
		{
			return 2;
		}
		else  // MASTER
		{
			return 0;
		}
	}

	return 3;
}
