#include "MapCommon.as";

namespace TWMap
{
	const bool movableCrates = false;

	void PlaceAbove(CMap@ map, int offset, u32 tile)
	{
		int aboveoffset = offset - map.tilemapwidth;
		if (aboveoffset > 0 && map.getTile(aboveoffset).type == 0)
		{
			map.SetTile(aboveoffset, tile);
		}
	}

	void HandlePixel(CMap@ map, SColor pixel, int offset)
	{
		int row = (offset / map.tilemapwidth) % 2;
		int row_off = (map.tilemapwidth + 1) % 2;
		bool dither = (((row * row_off) + (offset % 2)) % 2) == 0;
		Random tilerandom(offset);

		if (pixel == color_bgdecor)
		{
			map.SetTile(offset, tile_bgdecor_panel);
		}
		else if (pixel == color_bgdecor_room)
		{
			map.SetTile(offset, tile_roofdecor_1);
		}
		else if (pixel == color_dirt)
		{
			map.SetTile(offset, tile_dirt_1);
		}
		else if (pixel == color_dirt_grass)
		{
			map.SetTile(offset, tile_dirt_grass);
		}
		else if (pixel == color_sand)
		{
			map.SetTile(offset, tile_sand_1);
		}
		else if (pixel == color_dark_solid)
		{
			map.SetTile(offset, tile_dark_solid);
		}
		else if (pixel == color_bush)
		{
			map.SetTile(offset, tile_bush);
		}
		else if (pixel == color_bgbushdecor_leaves)
		{
			map.SetTile(offset, tile_bgbushdecor_leaves);
		}
		else if (pixel == color_bgbushdecor_trunk)
		{
			map.SetTile(offset, tile_bgbushdecor_trunk);
		}
		else if (pixel == color_bgbushdecor_branch)
		{
			map.SetTile(offset, tile_bgbushdecor_branch);
		}
		else if (pixel == color_bunker)
		{
			map.SetTile(offset, tile_bunker);
		}
		else if (pixel == color_bunker_wall)
		{
			map.SetTile(offset, tile_bunker_up);
		}
		else if (pixel == color_bunker_wood)
		{
			map.SetTile(offset, tile_bunker_wood_up);
		}
		else if (pixel == color_rubble)
		{
			map.SetTile(offset, tile_rubble);
		}
		else if (pixel == color_rubble_background)
		{
			map.SetTile(offset, tile_rubble_background);
		}
		else if (pixel == color_woodbridge)
		{
			map.SetTile(offset, tile_woodbridge);
		}
		else if (pixel == color_crate)
		{
			map.SetTile(offset, tile_crate_1 + tilerandom.NextRanged(4));
		}
		else if (pixel == color_roof)
		{
			map.SetTile(offset, tile_roof_body);
		}
		else if (pixel == color_reeds)
		{
			map.SetTile(offset, tile_reeds_1 + tilerandom.NextRanged(4));
		}
		else if (pixel == color_cactus)
		{
			map.SetTile(offset, tile_cactus_1 + (dither ? 1 : 0));
		}
		else if (pixel == color_flower)
		{
			map.SetTile(offset, tile_flower_1);
		}
		else if (pixel == color_water)
		{
			map.SetTile(offset, tile_black_background);
			map.server_setFloodWaterOffset(offset, true);
		}
		else if (pixel == color_swamp_bed)
		{
			map.SetTile(offset, tile_swamp_bed_1);
		}
		else if (pixel == color_ladder)
		{
			map.SetTile(offset, tile_ladder);
		}
		else if (pixel == color_mountstone)
		{
			map.SetTile(offset, tile_mountstone_1);
		}
		else if (pixel == color_snow)
		{
			map.SetTile(offset, tile_snow_1);
		}
		else if (pixel == color_black_background)
		{
			map.SetTile(offset, tile_black_background);
		}
		//spawns
		else if (pixel == color_neutral_spawn)
		{
			AddMarker(map, offset, "neutral spawn");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_blue_spawn)
		{
			AddMarker(map, offset, "blue spawn");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_red_spawn)
		{
			AddMarker(map, offset, "red spawn");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_green_spawn)
		{
			AddMarker(map, offset, "green spawn");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_pink_spawn)
		{
			AddMarker(map, offset, "pink spawn");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_blue_sniperspot)
		{
			AddMarker(map, offset, "blue sniper spot");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_red_sniperspot)
		{
			AddMarker(map, offset, "red sniper spot");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_classchange)
		{
			AddMarker(map, offset, "class change");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_campaignview)
		{
			AddMarker(map, offset, "campaign view");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_bottle)
		{
			AddMarker(map, offset, "bottle");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_band)
		{
			AddMarker(map, offset, "band");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_skirmish_truck)
		{
			AddMarker(map, offset, "skirmish truck");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_blue_truck)
		{
			AddMarker(map, offset, "blue truck");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_red_truck)
		{
			AddMarker(map, offset, "red truck");
			map.SetTile(offset, tile_placemostlikely);
		}
		else if (pixel == color_croc)
		{
			AddMarker(map, offset, "croc");
			map.SetTile(offset, tile_placemostlikely);
		}
		else
		{
			// empty
		}
	}

	void PostLoad(CMap@ map)
	{
		Vec2f mapdim = map.getMapDimensions();

		// crates

		if (movableCrates)
		{
			for (uint y = map.tilemapheight - 2; y > 0 ; y--) //loop from bottom to top for more sensible autotile
			{
				for (uint x = 1; x < map.tilemapwidth - 1; x++)
				{
					u32 offset = offsetAt(map, x, y);
					u8 type = map.getTile(offset).type;

					if (type == tile_crate_1 || type == tile_crate_2)
					{
						bool up    = isTileTypeSolid(getTile(x, y - 1));
						bool left  = isTileTypeSolid(getTile(x - 1, y));
						bool right = isTileTypeSolid(getTile(x + 1, y));
						bool down  = isTileTypeSolid(getTile(x, y + 1));
						if (!left && !right)
						{
							map.SetTile(offset, tile_placemostlikely);
							CBlob@ crate = server_CreateBlob("falling_tile", -1, Vec2f(x + 0.5f, y + 0.5f) * map.tilesize);
							if (crate !is null)
							{
								crate.Tag("resting");
							}
						}
					}
				}
			}
		}

		//first pass - build depth map
		array<u32> depthmap(map.tilemapwidth * map.tilemapheight, 0);
		array<u32> depth_todo;
		for (uint y = 0; y < map.tilemapheight; y++)
		{
			for (uint x = 0; x < map.tilemapwidth; x++)
			{
				u32 offset = offsetAt(map, x, y);
				u8 type = map.getTile(offset).type;
				depthmap[offset] = 0;
				if (isTileSolidForAutoTile(type))
				{
					if (isTileForDepth(type))
					{
						depthmap[offset] = 1;
						depth_todo.push_back(offset);
					}
					else //fake wall
					{
						depthmap[offset] = 10;
					}
				}
			}
		}
		while (!depth_todo.empty())
		{
			u32 offset = depth_todo[0];
			u32 x = offset % map.tilemapwidth;
			u32 y = offset / map.tilemapwidth;
			depth_todo.removeAt(0);

			u32 left = (x > 0 ? depthmap[offset - 1] : 10);
			u32 right = (x < map.tilemapwidth - 1 ? depthmap[offset + 1] : 10);
			u32 up = (y > 0 ? depthmap[offset - map.tilemapwidth] : 10);
			u32 down = (y < map.tilemapheight - 1 ? depthmap[offset + map.tilemapwidth] : 10) + 1;

			u32 depth = Maths::Min(Maths::Min(left, right), Maths::Min(up, down)) + 1;

			if (depth != depthmap[offset])
			{
				depthmap[offset] = depth;
				depth_todo.push_back(offset);
			}
		}

		// autotile
		Random tilerandom(0xf4ce);
		Noise tilenoise(tilerandom.Next());
		for (uint y = map.tilemapheight - 2; y > 0; y--) //loop from bottom to top for more sensible autotile
		{
			for (uint x = 1; x < map.tilemapwidth - 1; x++)
			{
				u32 offset = offsetAt(map, x, y);
				int row = (offset / map.tilemapwidth) % 2;
				int row_off = (map.tilemapwidth + 1) % 2;
				bool dither = (((row * row_off) + (offset % 2)) % 2) == 0;
				Random tilerandom(offset);
				u8 type = map.getTile(offset).type;

				if (type == tile_placemostlikely)
				{
					PlaceMostLikelyTile(map, offset, true);
					continue;
				}

				//autotiling into space
				if (type == 0)
				{
					u8 tile_up    = getTile(x, y - 1);
					u8 tile_down  = getTile(x, y + 1);
					u8 tile_left  = getTile(x - 1, y);
					u8 tile_right = getTile(x + 1, y);

					bool rough_up    = false;
					bool rough_down  = false;
					bool rough_left  = false;
					bool rough_right = false;

					//count rough edges
					u32 rough_edges = 0;
					if (x > 0 && isTileForRough(tile_left))
					{
						rough_edges++;
						rough_left = true;
					}
					if (x < map.tilemapwidth - 1 && isTileForRough(tile_right))
					{
						rough_edges++;
						rough_right = true;
					}
					if (y > 0 && isTileForRough(tile_up))
					{
						rough_edges++;
						rough_up   = true;
					}
					if (y < map.tilemapheight - 1 && isTileForRough(tile_down))
					{
						rough_edges++;
						rough_down = true;
					}

					if (rough_edges == 1)
					{
						setTile(x, y, tile_rough_backedge);
					}
					else if (rough_edges == 2)
					{
						if (rough_up && rough_down || rough_left && rough_right)
							setTile(x, y, tile_rough_tight);
						else
							setTile(x, y, tile_rough_corner);
					}
					else if (rough_edges == 3)
					{
						setTile(x, y, tile_rough_blind);
					}
					else if (isTileRoof(tile_down))
					{
						if (isTileRoof(tile_right))
						{
							setTile(x, y, tile_bgdecor_ramp_left);
						}
						if (isTileRoof(tile_left))
						{
							setTile(x, y, tile_bgdecor_ramp_right);
						}
					}
					else if (isTileSand(tile_down))
					{
						if (isTileSand(tile_left) || isTileSand(tile_right))
						{
							setTile(x, y, tile_sand_decor_curve);
						}
						else
						{
							setTile(x, y, tile_sand_decor_1 + (dither ? 1 : 0));
						}
					}
					else if (isTileSnow(tile_down))
					{
						setTile(x, y, tile_snow_over_1 + (dither ? 1 : 0));
					}
					else if (tile_down == tile_bgbushdecor_trunk)
					{
						setTile(x, y, tile_bgbushdecor_trunk_top);
					}
					continue;
				}

				if (type == tile_roofdecor_1)
				{
					u8 tile_up     = getTile(x, y - 1);
					u8 tile_down   = getTile(x, y + 1);
					u8 tile_left   = getTile(x - 1, y);
					u8 tile_right  = getTile(x + 1, y);

					u8 tile_two_up     = getTile(x, y - 2);
					u8 tile_two_down   = getTile(x, y + 2);
					u8 tile_two_left   = getTile(x - 2, y);
					u8 tile_two_right  = getTile(x + 2, y);

					if (isTileBunker(tile_up) || isTileWoodBridge(tile_up) ||
					        (isTileAutoDecor(tile_up) && (tile_two_up == 0 || tile_two_up == tile_black_background)))
					{
						type = tile_roofdecor_1 + tilerandom.NextRanged(2);
					}
					else if (tile_up == tile_roofdecor_1 && (isTileBunker(tile_two_up) || isTileWoodBridge(tile_two_up)))
					{
						type = (tilerandom.NextRanged(2) == 0) ? tile_bgdecor_handle_top : tile_black_background;
					}
					else if ((isTileBunker(tile_down) || isTileWoodBridge(tile_down)) && tilenoise.Sample(x, y) > 0.8f)
					{
						type = tile_bgdecor_crate;
					}
					else
					{
						type = tile_black_background;
					}

					setTile(x, y, type);

					continue;
				}

				if (type == tile_bgdecor_panel)
				{
					u8 tile_up     = getTile(x, y - 1);
					u8 tile_down   = getTile(x, y + 1);
					u8 tile_left   = getTile(x - 1, y);
					u8 tile_right  = getTile(x + 1, y);

					u8 tile_two_up     = getTile(x, y - 2);
					u8 tile_two_down   = getTile(x, y + 2);
					u8 tile_two_left   = getTile(x - 2, y);
					u8 tile_two_right  = getTile(x + 2, y);

					//figure out some way to do the roof decor

					bool door_vert = isTileSolidForAutoTile(tile_up) && isTileSolidForAutoTile(tile_two_down) ||
					                 isTileSolidForAutoTile(tile_down) && isTileSolidForAutoTile(tile_two_up);

					bool door_horiz = isTileSolidForAutoTile(tile_left) && isTileSolidForAutoTile(tile_two_right) ||
					                  isTileSolidForAutoTile(tile_right) && isTileSolidForAutoTile(tile_two_left);

					u32 count_up = 0;
					if (isTileAutoDecor(tile_up)) {count_up++;}
					if (isTileAutoDecor(tile_two_up)) {count_up++;}

					u32 count_down = 0;
					if (isTileAutoDecor(tile_down)) {count_down++;}
					if (isTileAutoDecor(tile_two_down)) {count_down++;}

					if (door_vert) {count_up++; count_down++;}

					u32 count_left = 0;
					if (isTileAutoDecor(tile_left)) {count_left++;}
					if (isTileAutoDecor(tile_two_left)) {count_left++;}

					u32 count_right = 0;
					if (isTileAutoDecor(tile_right)) {count_right++;}
					if (isTileAutoDecor(tile_two_right)) {count_right++;}

					if (door_horiz) {count_left++; count_right++;}

					if (count_left + count_right + count_up + count_down > 0)
					{
						if (count_left + count_right >= count_up + count_down)
						{
							type = door_horiz ? tile_bgdecor_door_horiz : tile_bgdecor_girder_horiz;
						}
						else
						{
							type = door_vert ? tile_bgdecor_door_vert : tile_bgdecor_girder_vert;
						}
					}

					setTile(x, y, type);
					continue;
				}

				//auto-border decor placement placement "indoors"
				if (type == tile_black_background)
				{
					if (y > 0 && x > 0 && y < map.tilemapheight - 1 && x < map.tilemapwidth - 1)
					{
						u8 above = getTile(x, y - 1);
						u8 below = getTile(x, y + 1);
						u8 left = getTile(x - 1, y);
						u8 right = getTile(x + 1, y);
						if (below == tile_dirt_grass)
						{
							setTile(x, y, tile_grass_short_cave);
						}

						//todo: corners
						if (isTileDirt(below))
						{
							if (isTileDirt(left) || isTileDirt(right))
								setTile(x, y, tile_dirtdecor_corner_bottom);
							else
								setTile(x, y, tile_dirtdecor_floor_1 + (dither ? 1 : 0));
						}
						else if (isTileDirt(above))
						{
							if (isTileDirt(left) || isTileDirt(right))
								setTile(x, y, tile_dirtdecor_corner_top);
							else
								setTile(x, y, tile_dirtdecor_roof_1 + (dither ? 1 : 0));
						}
						else if (isTileDirt(left) || isTileDirt(right))
						{
							setTile(x, y, tile_dirtdecor_wall);
						}

						if (isTileSand(left) || isTileSand(right))
						{
							if (isTileSand(above) || isTileSand(below))
								setTile(x, y, tile_sand_decor_curve_dark);
							else
								setTile(x, y, tile_sand_dark_wall);
						}
						else if (isTileSand(below))
						{
							setTile(x, y, tile_sand_dark_floor);
						}
						else if (isTileSand(above))
						{
							setTile(x, y, tile_sand_dark_roof);
						}

						if (isTileMountainStone(below))
						{
							setTile(x, y, tile_mountstone_over_bg_1 + (dither ? 1 : 0));
						}
						else if (isTileMountainStone(above))
						{
							setTile(x, y, tile_mountstone_under_bg_1 + (dither ? 0 : 1));
						}
						else if (isTileMountainStone(left) || isTileMountainStone(right))
						{
							setTile(x, y, tile_mountstone_side_bg);
						}
					}
				}

				//bush autotiling
				if (type == tile_bush)
				{
					bool up    = !isTileTypeBackground(getTile(x, y - 1));
					bool down  = !isTileTypeBackground(getTile(x, y + 1));
					bool left  = !isTileTypeBackground(getTile(x - 1, y));
					bool right = !isTileTypeBackground(getTile(x + 1, y));

					if (up && down && left && right)
					{
						type = tile_bush_light_1 + (dither ? 1 : 0);
					}
					else if (up != down && !left && !right ||
					         !up && !down && left != right)
					{
						type = tile_bush_single;
					}
					else if (up != down && left != right)
					{
						type = tile_bush_corner;
					}
					map.SetTile(offset, type);
				}

				//flower autotiling
				if (type == tile_flower_1)
				{
					if (getTileBackgroundAt(map, offset) != 0)
						type += 2;

					type += (dither ? 1 : 0);
					map.SetTile(offset, type);
					continue;
				}

				//building autotiling
				if (type == tile_roof_body)
				{
					//TODO MAP cleverer sloping here
					if (!isTileRoof(getTile(x, y - 1)))
					{
						type = tile_roof_top;
					}
					map.SetTile(offset, type);
					continue;
				}


				//terrain tiles
				if (isTileForDepth(type))
				{
					f32 depth = 0.5f;

					depth += depthmap[offset] - 1.0f;// calculateDepthAt(map, x, y, tilenoise);

					if (type == tile_dirt_1 || type == tile_mountstone_1)
					{
						depth = Maths::Max(0, depth - tilenoise.Sample(x, y));
						if (depth >= 3.5f)
							type = tile_dark_solid;
						else
							type += u32(depth) * 2 + (dither ? 1 : 0);
					}
					else if (type == tile_sand_1)
					{
						int[] offsets = {0, 1, 2, 2};
						if (depth >= offsets.length)
						{
							type = tile_dark_solid;
						}
						else
						{
							int off = offsets[u32(depth)];
							if (off != 0 || tilerandom.NextRanged(2) == 0)
								type += 2;
							type += off * 2 + (dither ? 1 : 0);
						}
					}
					else if (type == tile_snow_1)
					{
						depth = Maths::Max(0.0f, depth);
						if (depth >= 2.5f)
							type = tile_dark_solid;
						if (depth < 1.0f)
							type = (tilerandom.NextRanged(2) == 0 ? tile_snow_1 : tile_snow_3) + (dither ? 1 : 0);
						else
							type = tile_snow_5 + (dither ? 1 : 0);
					}

					map.SetTile(offset, type);
				}

				if (type == tile_woodbridge)
				{
					u8 tile_up    = getTile(x, y - 1);
					u8 tile_down  = getTile(x, y + 1);
					u8 tile_left  = getTile(x - 1, y);
					u8 tile_right = getTile(x + 1, y);

					bool up    = isTileWoodBridge(tile_up) || isTileBunker(tile_up);
					bool down  = isTileWoodBridge(tile_down) || isTileBunker(tile_down);
					bool left  = isTileWoodBridge(tile_left);
					bool right = isTileWoodBridge(tile_right);

					if (up || down)
					{
						if (!up)
						{
							type = tile_woodbridge_top;
						}
						else if (left || right)
						{
							type = (down ? tile_woodbridge_join : tile_woodbridge);
						}
						else if (!down)
						{
							type = tile_woodbridge_bottom;
						}
						else
						{
							type = tile_woodbridge_vert;
						}
					}
					map.SetTile(offset, type);
				}

				//final directional autotiles
				if (type != tile_bunker_up &&
				        type != tile_bunker_wood_up)
				{
					continue;
				}

				bool up    = isTileSolidForAutoTile(getTile(x, y - 1));
				bool left  = isTileSolidForAutoTile(getTile(x - 1, y));
				bool right = isTileSolidForAutoTile(getTile(x + 1, y));
				bool down  = isTileSolidForAutoTile(getTile(x, y + 1));

				switch (type)
				{
					//concrete bunker wall auto-tile
					case tile_bunker_up:
						if (!up)
							type = tile_bunker_up;
						else if (!down)
							type = tile_bunker_down;
						else if (!left)
							type = tile_bunker_left;
						else if (!right)
							type = tile_bunker_right;

						map.SetTile(offset, type);
						break;

					//concrete+wood bunker wall auto-tile
					case tile_bunker_wood_up:
						if (!up)
							type = tile_bunker_wood_up;
						else if (!down)
							type = tile_bunker_wood_down;
						else if (!left)
							type = tile_bunker_wood_left;
						else if (!right)
							type = tile_bunker_wood_right;
						map.SetTile(offset, type);
						break;
				}
			}
		}
	}

	f32 calculateDepthAt(CMap@ map, s32 x, s32 y, Noise@ noise)
	{
		f32 depth = 0.0f;
		bool blockedup    = false;
		bool blockedleft  = false;
		bool blockedright = false;
		bool blockeddown  = false;
		for (u32 i = 1; i <= 5; i++)
		{
			u8 tileup = getTile(x, y - i);
			u8 tileleft = getTile(x - i, y - i / 2);
			u8 tileright = getTile(x + i, y - i / 2);
			u8 tiledown = getTile(x, y + i / 2);

			if (isTileBunker(tileup)) blockedup = true;
			if (isTileBunker(tiledown)) blockeddown = true;
			if (isTileBunker(tileleft)) blockedleft = true;
			if (isTileBunker(tileright)) blockedright = true;

			bool up    = blockedup    || isTileSolidForAutoTile(tileup);
			bool left  = blockedleft  || (x - i > 0 ? isTileSolidForAutoTile(tileleft) : true);
			bool right = blockedright || (x + i < map.tilemapwidth - 1 ? isTileSolidForAutoTile(tileright) : true);
			bool down  = blockeddown  || (y + i < map.tilemapheight - 1 ? isTileSolidForAutoTile(tiledown) : true);
			if (!up || !left || !right || !down)
			{
				break;
			}
			depth++;
		}
		return depth;
	}

}
